/*
Copyright (C) 2019 RRe36

All Rights Reserved unless otherwise explicitly stated.


By downloading this you have agreed to the license and terms of use.
These can be found inside the included license-file or here: https://rre36.github.io/kappa_shader_web/license/

Violating these terms may be penalized with actions according to the Digital Millennium Copyright Act (DMCA), the Information Society Directive and/or similar laws depending on your country.
*/

#include "/lib/common.glsl"
#include "/lib/util/srgb.glsl"
#include "/lib/util/encoders.glsl"

in mat2x2 coord;

in vec4 tint;

flat in vec3 normal;

#ifdef g_textured
    uniform sampler2D gcolor;

    #ifdef labpbr_enabled
        uniform sampler2D specular;
    #endif

    #if (defined labpbr_enabled || defined normalmap_enabled)
        uniform sampler2D normals;

        flat in mat3x3 tbn;

        #define ENABLE_NRM_POM_PROCESSING
    #endif
#endif

float encodeMatID16(int x) {
    float id    = float(x)/65535.0;
    return id;
}

#ifdef g_terrain
    flat in int foliage;
    flat in int mat_id;

    #ifdef pom_enabled
        /* */
    #endif

    #if (defined ENABLE_NRM_POM_PROCESSING && defined pom_enabled)
        in float vert_dist;
        in vec2 vcoord;
        in vec4 vcoord_a;
        in vec3 viewvec;

        #include "/lib/frag/parallax.glsl"

        #define ENABLE_POM_SAMPLING
    #endif
#endif

#ifdef g_entity
uniform vec4 entityColor;
#endif

#ifdef g_textured
    #ifdef labpbr_enabled
        vec3 decode_lab_nrm(vec3 ntex, inout float ao) {
            vec3 nrm    = ntex * 2.0 - (254.0 * rcp(255.0));

            #if normalmap_format==0
                ao     *= ntex.z;
                nrm.z  = sqrt(saturate(1.0 - dot(nrm.xy, nrm.xy)));
            #elif normalmap_format==1
                ao     *= pow2(length(nrm));
                nrm    = normalize(nrm);
            #endif

            return normalize(nrm * tbn);
        }
    #else
        #ifdef normalmap_enabled
        vec3 decode_nrm(vec3 ntex) {
            vec3 nrm    = ntex * 2.0 - (254.0 * rcp(255.0));

            #if normalmap_format==0
                nrm.z  = sqrt(saturate(1.0 - dot(nrm.xy, nrm.xy)));
            #elif normalmap_format==1
                nrm    = normalize(nrm);
            #endif

            return normalize(nrm * tbn);
        }
        #endif
    #endif
#endif

void main() {
    #ifdef hide_terrain
        discard;
    #endif

    float return2_y     = 1.0;

    #ifdef g_textured
        vec4 scenecol   = texture(gcolor, coord[0]);
        if (scenecol.a<0.1) discard;
        vec3 scenenormal = normal;

            scenecol.rgb *= tint.rgb;
        
        #ifdef g_terrain
            scenecol.a  = tint.a;
        #else
            scenecol.a  = 1.0;
        #endif

        #ifdef g_entity
            scenecol.rgb = mix(scenecol.rgb, entityColor.rgb, entityColor.a);
        #endif

        #ifdef labpbr_enabled
            #ifdef ENABLE_POM_SAMPLING
                mat2 dcoord     = get_dcoord(coord[0]);
                vec2 r_coord;
                vec2 p_coord    = compute_parallax_coord(coord[0], dcoord, r_coord);

                scenecol.rgb    = textureParallax(gcolor, p_coord, dcoord).rgb * tint.rgb;
                vec4 spectex    = textureParallax(specular, p_coord, dcoord);
                vec4 ntex       = textureParallax(normals, p_coord, dcoord);
            #else
                vec4 spectex    = texture(specular, coord[0]);
                vec4 ntex       = texture(normals, coord[0]);
            #endif

            vec2 return1_zw = vec2(encode2x8(spectex.xy), encode2x8(spectex.zw));

            scenenormal     = decode_lab_nrm(ntex.rgb, scenecol.a);

            #if (defined ENABLE_POM_SAMPLING && defined pom_shadows_enabled)
                return2_y   = compute_parallax_shadow(scenenormal, p_coord, r_coord, dcoord, ntex.a);
            #endif
        #else
            #ifdef normalmap_enabled
                #ifdef ENABLE_POM_SAMPLING
                    mat2 dcoord     = get_dcoord(coord[0]);
                    vec2 r_coord;
                    vec2 p_coord    = compute_parallax_coord(coord[0], dcoord, r_coord);

                    scenecol.rgb    = textureParallax(gcolor, p_coord, dcoord).rgb * tint.rgb;
                    vec4 ntex       = textureParallax(normals, p_coord, dcoord);

                    #if (defined ENABLE_POM_SAMPLING && defined pom_shadows_enabled)
                        return2_y   = compute_parallax_shadow(scenenormal, p_coord, r_coord, dcoord, ntex.a);
                    #endif
                #else
                    vec4 ntex       = texture(normals, coord[0]);
                #endif

                scenenormal     = decode_nrm(ntex.rgb);
            #endif

            const vec2 return1_zw = vec2(1.0);
        #endif

    #else
        vec4 scenecol   = tint;
        if (scenecol.a<0.01) discard;
        vec3 scenenormal = normal;
        
            scenecol.a  = 1.0;

        const vec2 return1_zw = vec2(1.0);
    #endif

        scenecol.rgb    = to_linear(scenecol.rgb);

    #ifdef g_terrain
        int matID   = mat_id;
        //if (foliage==1) matID = 2;
    #else
        #ifdef g_nodiff
        const int matID = 2;
        #else
        const int matID = 1;
        #endif
    #endif

    /*DRAWBUFFERS:012*/
    gl_FragData[0]  = make_drawbuffer(scenecol.rgb, saturate(scenecol.a));
    gl_FragData[1]  = vec4(encode2x8(encodeNormal(scenenormal)), encode2x8(coord[1]), return1_zw);
    gl_FragData[2]  = vec4(encodeMatID16(matID), return2_y, 0.0, 0.0);
}