/*
Copyright (C) 2019 RRe36

All Rights Reserved unless otherwise explicitly stated.


By downloading this you have agreed to the license and terms of use.
These can be found inside the included license-file or here: https://rre36.github.io/kappa_shader_web/license/

Violating these terms may be penalized with actions according to the Digital Millennium Copyright Act (DMCA), the Information Society Directive and/or similar laws depending on your country.
*/

#define blocklight_luma 1.0     //[0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0 1.1 1.2 1.3 1.4 1.5 1.6 1.7 1.8 1.9 2.0 2.5 3.0 4.0 5.0]
#define blocklight_temp 2000.0  //[1000.0 1500.0 2000.0 2500.0 3000.0 3500.0 4000.0 4500.0 5000.0 5500.0 6000.0 6500.0 7000.0]
#define moon_illumination_mult 1.0  //[0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0 1.1 1.2 1.3 1.4 1.5 1.6 1.7 1.8 1.9 2.0 2.5 3.0 4.0 5.0]

uniform float eyeAltitude;

uniform vec4 daytime;
uniform vec2 viewSize;
uniform vec3 sunvec;

flat out float light_flip;

flat out mat4x3 light_color;

uniform sampler2D gaux2;    //colortex5

#ifdef IBL_pass
flat out mat4x3 IBL_light;
/*
flat out mat2x3 IBL_e_1;
flat out mat2x3 IBL_e_2;
flat out mat2x3 IBL_e_3;

flat out mat2x3 IBL_w_1;
flat out mat2x3 IBL_w_2;
flat out mat2x3 IBL_w_3;*/
#endif

vec3 blackbody(float temperature){
    vec4 vx = vec4(-0.2661239e9,-0.2343580e6,0.8776956e3,0.179910);
    vec4 vy = vec4(-1.1063814,-1.34811020,2.18555832,-0.20219683);
    float it = 1./temperature;
    float it2= it*it;
    float x = dot(vx,vec4(it*it2,it2,it,1.));
    float x2 = x*x;
    float y = dot(vy,vec4(x*x2,x2,x,1.));
    float z = 1. - x - y;
    
    mat3 xyzToSrgb = mat3(
         3.2404542,-1.5371385,-0.4985314,
        -0.9692660, 1.8760108, 0.0415560,
         0.0556434,-0.2040259, 1.0572252
    );
    vec3 srgb = vec3(x/y,1.,z/y) * xyzToSrgb;
    return max(srgb,0.);
}

uniform float wetness;

#include "/lib/atmos/aconst.glsl"

#include "/lib/atmos/arho.glsl"

#include "/lib/atmos/project.glsl"

#ifdef IBL_pass
vec3 get_projected_sky(vec3 x) {
    return texture(gaux2, projectSky(x)).rgb;
}
#endif

void make_colors() {
    /*
    vec3 sunlightSunrise;
        sunlightSunrise.r   = 1.00;
        sunlightSunrise.g   = 0.3;
        sunlightSunrise.b   = 0.00;
        sunlightSunrise    *= 1.5;

    vec3 sunlightNoon;
        sunlightNoon.r      = 1.00;
        sunlightNoon.g      = 0.95;
        sunlightNoon.b      = 0.94;
        sunlightNoon       *= 1.5;

    vec3 sunlightSunset;
        sunlightSunset.r    = 1.00;
        sunlightSunset.g    = 0.26;
        sunlightSunset.b    = 0.01;
        sunlightSunset     *= 0.9;

    vec3 sunlightNight;
        sunlightNight.r     = 1.00;
        sunlightNight.g     = 0.22;
        sunlightNight.b     = 0.00;
        sunlightNight      *= 0.6;

    light_color[0]  = sunlightSunrise*daytime.x + sunlightNoon*daytime.y + sunlightSunset*daytime.z + sunlightNight*daytime.w;*/
    /*
    float stemp     = 1940.0*daytime.x + 5775.0*daytime.y + 1942.0*daytime.z + 1465.0*daytime.w;
    light_color[0]  = blackbody(stemp);
    light_color[0] *= sunlight_luma;*/
    light_color[0]  = sun_illum * air_transmittance(vec3(0.0, planetRad + eyeAltitude, 0.0), sunvec, 5) * (1.0-wetness*0.5);
    light_color[1]  = texture(gaux2, projectSky(vec3(0.0, 1.0, 0.0))).rgb*pi;

    light_color[2]  = blackbody(blocklight_temp) * 2.0 * blocklight_luma;

    light_color[3]  = vec3(0.5, 0.7, 1.0)* moon_illumination_mult * 0.04 * (1.0-wetness*0.5);

    float lf    = dot(sunvec, vec3(0.0, 1.0, 0.0))*0.5+0.5;
        lf      = lin_step(lf, 0.48, 0.499)*(1.0-lin_step(lf, 0.501, 0.52));

    light_flip  = saturate(1.0-lf);

    #ifdef IBL_pass
        mat2x3 IBL_e_1;
        mat2x3 IBL_e_2;
        mat2x3 IBL_e_3;

        IBL_e_1[0]  = get_projected_sky(normalize(vec3(1.0, 0.15, -0.4)));
        IBL_e_1[1]  = get_projected_sky(normalize(vec3(1.0, 0.40, -0.4)));
        IBL_e_2[0]  = get_projected_sky(normalize(vec3(1.0, 0.15,  0.0)));
        IBL_e_2[1]  = get_projected_sky(normalize(vec3(1.0, 0.40,  0.0)));
        IBL_e_3[0]  = get_projected_sky(normalize(vec3(1.0, 0.15,  0.4)));
        IBL_e_3[1]  = get_projected_sky(normalize(vec3(1.0, 0.40,  0.4)));

        IBL_light[0] = IBL_e_1[0] + IBL_e_1[1] + IBL_e_2[0] + IBL_e_2[1] + IBL_e_3[0] + IBL_e_3[1];

        mat2x3 IBL_s_1;
        mat2x3 IBL_s_2;
        mat2x3 IBL_s_3;

        IBL_s_1[0]  = get_projected_sky(normalize(vec3(-0.4, 0.15,  1.0)));
        IBL_s_1[1]  = get_projected_sky(normalize(vec3(-0.4, 0.40,  1.0)));
        IBL_s_2[0]  = get_projected_sky(normalize(vec3( 0.0, 0.15,  1.0)));
        IBL_s_2[1]  = get_projected_sky(normalize(vec3( 0.0, 0.40,  1.0)));
        IBL_s_3[0]  = get_projected_sky(normalize(vec3( 0.4, 0.15,  1.0)));
        IBL_s_3[1]  = get_projected_sky(normalize(vec3( 0.4, 0.40,  1.0)));

        IBL_light[1] = IBL_s_1[0] + IBL_s_1[1] + IBL_s_2[0] + IBL_s_2[1] + IBL_s_3[0] + IBL_s_3[1];

        mat2x3 IBL_w_1;
        mat2x3 IBL_w_2;
        mat2x3 IBL_w_3;

        IBL_w_1[0]  = get_projected_sky(normalize(vec3(-1.0, 0.15, -0.4)));
        IBL_w_1[1]  = get_projected_sky(normalize(vec3(-1.0, 0.40, -0.4)));
        IBL_w_2[0]  = get_projected_sky(normalize(vec3(-1.0, 0.15,  0.0)));
        IBL_w_2[1]  = get_projected_sky(normalize(vec3(-1.0, 0.40,  0.0)));
        IBL_w_3[0]  = get_projected_sky(normalize(vec3(-1.0, 0.15,  0.4)));
        IBL_w_3[1]  = get_projected_sky(normalize(vec3(-1.0, 0.40,  0.4)));

        IBL_light[2] = IBL_w_1[0] + IBL_w_1[1] + IBL_w_2[0] + IBL_w_2[1] + IBL_w_3[0] + IBL_w_3[1];
    
        mat2x3 IBL_n_1;
        mat2x3 IBL_n_2;
        mat2x3 IBL_n_3;

        IBL_n_1[0]  = get_projected_sky(normalize(vec3(-0.4, 0.15, -1.0)));
        IBL_n_1[1]  = get_projected_sky(normalize(vec3(-0.4, 0.40, -1.0)));
        IBL_n_2[0]  = get_projected_sky(normalize(vec3( 0.0, 0.15, -1.0)));
        IBL_n_2[1]  = get_projected_sky(normalize(vec3( 0.0, 0.40, -1.0)));
        IBL_n_3[0]  = get_projected_sky(normalize(vec3( 0.4, 0.15, -1.0)));
        IBL_n_3[1]  = get_projected_sky(normalize(vec3( 0.4, 0.40, -1.0)));

        IBL_light[3] = IBL_n_1[0] + IBL_n_1[1] + IBL_n_2[0] + IBL_n_2[1] + IBL_n_3[0] + IBL_n_3[1];

        IBL_light *= rcp(6.0) * pi * 0.6;
    #endif
}