
uniform vec2 vc_cd;

const float vc_size         = 0.00042;
const float vc_highedge     = vcloud_alt + vcloud_depth;

#define vc_powder_K 33.0

float cloud_phase(float cos_theta, float g) {
    float a     = cloud_mie(cos_theta, 0.44*g)+cloud_mie_dumb(cos_theta, 0.89*g)*0.75;
    float b     = cloud_mie(cos_theta, -0.25*g) * 1.44 * cloud_bsfade;

    return mix(a, b, 0.38) + 0.01 * g;
}

float vc_coverage(in vec3 pos) {
    float erode_low     = 1.0-sstep(pos.y, vcloud_alt, vcloud_alt+vcloud_depth*0.13);
    float erode_high    = sstep(pos.y, vcloud_alt+vcloud_depth*0.16, vc_highedge);
    float fade_low      = sstep(pos.y, vcloud_alt, vcloud_alt+vcloud_depth*0.075);
    float fade_high     = 1.0-sstep(pos.y, vcloud_alt+vcloud_depth*0.65, vc_highedge);

    vec3 wind       = vec3(vcloud_tick, 0.0, vcloud_tick*0.6);

    pos            *= vc_size;
    pos            += wind*0.1;

    float shape     = noise_2d(pos.xz*0.028).b;
        shape      -= 0.48+vc_cd.x;
        shape      += wetness*0.15;

        shape      *= fade_low;
        shape      *= fade_high;
        shape      -= erode_low*0.15;
        shape      -= erode_high*0.45;

    return max(shape*1.8, 0.0);
}

float vc_shape(vec3 pos, float coverage) {
    float wfade     = sstep(pos.y, vcloud_alt, vcloud_alt+vcloud_depth*0.33)*0.5+0.5;

    float dfade     = 0.01 + sstep(pos.y, vcloud_alt, vcloud_alt+vcloud_depth*0.2)*0.2;
        dfade      += sstep(pos.y, vcloud_alt+vcloud_depth*0.1, vcloud_alt+vcloud_depth*0.55)*0.7;
        dfade      += sstep(pos.y, vcloud_alt+vcloud_depth*0.2, vcloud_alt+vcloud_depth*0.65)*4.0;
        dfade      += sstep(pos.y, vcloud_alt+vcloud_depth*0.4, vc_highedge-vcloud_depth*0.15)*3.0;

    vec3 wind       = vec3(vcloud_tick, 0.0, vcloud_tick*0.6);

    pos            *= vc_size;
    pos            += wind*0.1;

    float shape     = coverage;

    float d1        = 0.0;
    float n1        = 0.0;

    vec3 s1         = noise_3d(pos*0.33);

        n1         += s1.x;     d1 += 1.0;
        n1         += s1.y*0.2; d1 += 0.2;

        n1         /= d1;

        shape      -= (1.0-n1)*0.21*wfade;

    #ifndef LQ_PASS
    float n2        = 0.0;
    float d2        = 0.0;

    vec3 s2         = sqrt(noise_3d(pos*0.62 + n1*0.2 + wind*0.1));

        n2         += s2.x;          d2 += 1.0;
        n2         += s2.y*0.6;     d2 += 0.6;

        n2         /= d2;

        shape      -= (1.0-n2)*0.19*wfade;

    vec3 p2         = pos + (n1+n2)*0.12;
        shape      -= (1.0-fbm(p2*50.0, wind*0.2, 0.66, 2.8, 2))*0.066*(1.5-wfade);
    #else
        shape      -= 0.06;
    #endif

    return max(shape*dfade*vc_cd.y, 0.0);
}

float vc_directOD(in vec3 pos, const int steps) {
    vec3 dir    = cloud_lvec;

    float basestep = mix(150.0, 20.0, saturate(float(steps-1)/4.0));
    float exponent = mix(2.2, 1.1, saturate(float(steps-8)/8.0));

    float stepsize  = basestep;
    float prev_step = stepsize;

    float od = 0.0;
    for(int i = 0; i < steps; ++i, pos += dir * stepsize) {
        if(pos.y > vc_highedge || pos.y < vcloud_alt) continue;
        prev_step = stepsize;
        stepsize *= exponent;
        
        float coverage  = vc_coverage(pos);
        if(coverage <= 0.0) continue;
        float attenCoeff = vc_shape(pos, coverage);
        od += attenCoeff*prev_step;
    } 

    return od;
}
float vc_skyOD(in vec3 pos, const int steps) {
    vec3 dir    = vec3(0.0, 1.0, 0.0);

    float stepsize = (vcloud_depth / steps);
        stepsize  *= (1.0-lin_step(pos.y, vcloud_alt, vc_highedge))*0.9+0.1;

    float od = 0.0;
    for(int i = 0; i < steps; ++i, pos += dir * stepsize) {
        if(pos.y > vc_highedge || pos.y < vcloud_alt) break;
        float coverage  = vc_coverage(pos);
        if(coverage <= 0.0) continue;
        float attenCoeff = vc_shape(pos, coverage);
        od += attenCoeff*stepsize;
    } 

    return od;
}

float vc_bouncelight(vec3 pos) {
    const float high    = vcloud_alt+vcloud_depth*0.3;
    return pow3(1.0-sstep(pos.y, vcloud_alt, high));
}