/*
Copyright (C) 2019 RRe36

All Rights Reserved unless otherwise explicitly stated.


By downloading this you have agreed to the license and terms of use.
These can be found inside the included license-file or here: https://rre36.github.io/license/

Violating these terms may be penalized with actions according to the Digital Millennium Copyright Act (DMCA), the Information Society Directive and/or similar laws depending on your country.
*/

const mat2x3 metal_eta[8] = mat2x3[8](
    mat2x3(vec3(2.9114, 2.9497, 2.5845), vec3(3.0893, 2.9318, 2.7670)),     //iron
    mat2x3(vec3(0.18299, 0.42108, 1.3734), vec3(3.4242, 2.3459, 1.7704)),   //gold
    mat2x3(vec3(1.3456, 0.96521, 0.61722), vec3(7.4746, 6.3995, 5.3031)),   //aluminum
    mat2x3(vec3(3.1071, 3.1812, 2.3230), vec3(3.3314, 3.3291, 3.1350)),     //chrome
    mat2x3(vec3(0.27105, 0.67693, 1.3164), vec3(3.6092, 2.6248, 2.2921)),   //copper
    mat2x3(vec3(1.9100, 1.8300, 1.4400), vec3(3.5100, 3.4000, 3.1800)),     //lead
    mat2x3(vec3(2.3757, 2.0847, 1.8453), vec3(4.2655, 3.7153, 3.1365)),     //platinum
    mat2x3(vec3(0.15943, 0.14512, 0.13547), vec3(3.9291, 3.1900, 2.3808))   //silver
);

void decode_lab(in vec4 unpacked_tex, inout vec4 mat_data, inout float emission, inout bool is_metal, inout mat2x3 metal_e, inout bool hardcoded_metal) {
    mat_data.x  = pow2(1.0 - unpacked_tex.x);   //roughness
    mat_data.y  = pow2(unpacked_tex.y);         //f0

    unpacked_tex.z = round(unpacked_tex.z * 255.0);
    
    mat_data.z  = unpacked_tex.z < 64.5 ? lin_step(unpacked_tex.z, 0.0, 64.0) : 0.0;    //porosity
    mat_data.w  = lin_step(unpacked_tex.z, 64.0, 255.0);        //subsurface

    unpacked_tex.w = unpacked_tex.w * 255.0;

    emission    = unpacked_tex.w < 254.5 ? lin_step(unpacked_tex.w, 0.0, 254.0) : 0.0;

    is_metal    = (unpacked_tex.y * 255.0) > 229.5;

    int metal_index = clamp(int(unpacked_tex.y * 255.0 - 230.0), 0, 7);

    metal_e     = metal_eta[metal_index];

    hardcoded_metal = is_metal && ((unpacked_tex.y * 255.0) < 254.5);
}

void decode_lab(in vec4 unpacked_tex, inout vec4 mat_data, inout float emission, inout bool is_metal) {
    mat_data.x  = pow2(1.0 - unpacked_tex.x);   //roughness
    mat_data.y  = pow2(unpacked_tex.y);         //f0

    unpacked_tex.z = round(unpacked_tex.z * 255.0);
    
    mat_data.z  = unpacked_tex.z < 64.5 ? lin_step(unpacked_tex.z, 0.0, 64.0) : 0.0;    //porosity
    mat_data.w  = lin_step(unpacked_tex.z, 64.0, 255.0);        //subsurface

    unpacked_tex.w = unpacked_tex.w * 255.0;

    emission    = unpacked_tex.w < 254.5 ? lin_step(unpacked_tex.w, 0.0, 254.0) : 0.0;

    is_metal    = (unpacked_tex.y * 255.0) > 229.5;
}

vec3 compute_porosity(vec3 scenecol, vec3 data) {   //x-rough y-por z-wet
    vec3 wetcol    = color_saturation(pow2(scenecol), 0.85) * 0.95;

    return mix(scenecol, wetcol, data.y * pow2(data.x) * data.z);
}