/*
Copyright (C) 2019 RRe36

All Rights Reserved unless otherwise explicitly stated.


By downloading this you have agreed to the license and terms of use.
These can be found inside the included license-file or here: https://rre36.github.io/kappa_shader_web/license/

Violating these terms may be penalized with actions according to the Digital Millennium Copyright Act (DMCA), the Information Society Directive and/or similar laws depending on your country.
*/


mat2 get_dcoord(vec2 coord) {
    return mat2(dFdx(coord.xy), dFdy(coord.xy));
}

vec4 read_texture(sampler2D tex, vec2 coord, mat2 dcoord) {
    return textureGrad(tex, fract(coord) * vcoord_a.zw + vcoord_a.xy, dcoord[0], dcoord[1]);
}
vec4 textureParallax(sampler2D tex, vec2 coord, mat2 dcoord) {
    return textureGrad(tex, coord, dcoord[0], dcoord[1]);
}

vec2 compute_parallax_coord(vec2 coord, mat2 dcoord, out vec2 r_coord) {
    //mat2 dcoord     = get_dcoord(coord);
        //dcoord[0]   = dFdx(coord.xy);
        //dcoord[1]   = dFdy(coord.xy);

    vec2 p_coord    = vcoord * vcoord_a.zw + vcoord_a.xy;
        r_coord     = vcoord;

    float fade      = 1.0 - sstep(vert_dist, 24.0, 48.0);

    float parallax_depth = read_texture(normals, vcoord, dcoord).a;

    float rstep     = rcp(float(pom_iterations));

    const float mincoord = 1.0/(4096.0);

    if (fade > 0.0) {
        if (viewvec.z < 0.0 && parallax_depth < (1.0 - rstep)) {
            vec2 step   = viewvec.xy * 0.15 * fade * pom_depth * rcp(-viewvec.z * pom_iterations);

            for (int i = 0; i<pom_iterations; i++) {
                if (read_texture(normals, r_coord, dcoord).a < (1.0 - float(i) * rstep)) r_coord = r_coord + step;
                else break;
            }

            if (r_coord.y < mincoord && read_texture(gcolor, vec2(r_coord.x, mincoord), dcoord).a == 0.0) {
                r_coord.y = mincoord;
                discard;
            }
        }

        p_coord     = fract(r_coord) * vcoord_a.zw + vcoord_a.xy;
    }
    return p_coord;
}

uniform vec3 lightvec;

float compute_parallax_shadow(vec3 normal, vec2 p_coord, vec2 r_coord, mat2 dcoord, float height) {
    float shadow    = 1.0;

    float nDotL     = saturate(dot(normal, lightvec));

    float fade      = 1.0 - sstep(vert_dist, 24.0, 48.0);

    if (fade > 0.01 && nDotL > 0.01) {
        vec3 dir    = tbn * lightvec;
            dir     = dir;
            dir.xy *= 0.3 * pom_depth;
            float step = 1.28 / pom_shadow_iterations;

        for (int i = 0; i<pom_shadow_iterations; i++) {
            float curr_z    = height + dir.z * step * i;

            float height_o  = textureParallax(normals, fract(r_coord + dir.xy * i * step) * vcoord_a.zw + vcoord_a.xy, dcoord).a;
            shadow *= saturate(1.0 - (height_o - curr_z) * 40.0);
            if (shadow < 0.01) break;
        }
        shadow  = mix(1.0, shadow, fade);
    }
    return shadow;
}