/*
Copyright (C) 2019 RRe36

All Rights Reserved unless otherwise explicitly stated.


By downloading this you have agreed to the license and terms of use.
These can be found inside the included license-file or here: https://rre36.github.io/kappa_shader_web/license/

Violating these terms may be penalized with actions according to the Digital Millennium Copyright Act (DMCA), the Information Society Directive and/or similar laws depending on your country.
*/


vec3 shadow_scenespace(vec3 pos) {
        pos    = pos * 2.0 - 1.0;
        pos     = projMAD(shadowProjectionInverse, pos);
        pos     = viewMAD(shadowModelViewInverse, pos);

    return pos;
}

float dither_bluenoise_gi() {
    ivec2 coord = ivec2(fract(gl_FragCoord.xy/256.0)*256.0);
    float noise = texelFetch(noisetex, coord, 0).a;

    #ifdef taa_enabled
        noise   = fract(noise+float(frameCounter)/(pi * 0.5));
    #endif

    return noise;
}

vec2 circlemap(float i, float n){
	return sincos(i * n * golden_angle) * sqrt(i);
}

//based on the rsm in raspberry by rutherin https://www.patreon.com/user?u=12731730
vec2 fermatsSpiral_goldenS(float i, float total) {
	float theta = i * golden_angle;
	return vec2(sin(theta), cos(theta)) * pow2(i / total);
}
vec3 compute_rsm(vec3 scenenormal, vec3 shadowspace_position, vec3 scenepos, float cd, float skylight, inout bool sampled, bool reproject, bool offscreen) {
    int steps   = int(rsm_steps * 0.66);
    if (reproject) steps = int(rsm_steps * 1.33);
    //if (offscreen) steps = int(rsm_steps * 1.75);
    //const int steps = rsm_steps;

    const float dither_scale = 4096.0;
    float dither    = dither_scale * dither_bluenoise_gi() + 0.5;

    vec3 shadownormal = normalize(mat3(shadowModelView) * scenenormal);

    vec3 illum  = vec3(0.0);
    float weight = 1.0;

    float radius    = rsm_size * rcp(float(shadowMapResolution));

    sampled         = false;

    for (int i = 0; i < steps; i++) {
        vec2 offset     = fermatsSpiral_goldenS(i * dither_scale + dither, steps * dither_scale) * radius;
        //vec2 offset     = circlemap((float(i) + dither) * rcp(float(steps)), shadowMapResolution * 2.0 * steps) * radius;

        if (dot(shadownormal.xy, offset) <= 0.0) offset = -offset;

        vec3 offset_pos     = shadowspace_position + vec3(offset, 0.0);
        vec3 pos_warped     = offset_pos;
            pos_warped.xy   = warp_shadowmap(pos_warped.xy * 2.0 - 1.0) * 0.5 + 0.5;

        float shadowdepth   = texture(shadowtex1, pos_warped.xy).x;

        if ((shadowdepth * 256.0 + 0.01) < cd) {
            weight++;
            continue;
        }

        vec3 samplepos      = vec3(offset_pos.xy, shadowdepth);
        vec3 shadowpos_scene = shadow_scenespace(vec3(offset_pos.xy, shadowdepth));
        float samplelength  = distance(shadowpos_scene, samplepos);
            samplelength    = length(samplepos - shadowspace_position) * shadowMapResolution;

        //samplelength    = distance(a, scenepos);

        vec3 samplepos_norm     = normalize(samplepos - shadowspace_position) * vec3(1.0, 1.0, -1.0);

        if (samplelength > rsm_size) continue;

        weight++;
        /*
        float falloff   = rcp(1.0 + pow2(samplelength) * rcp(rsm_size) * 0.7);
            falloff    *= (1.0 - lin_step(samplelength, rsm_size*0.8, rsm_size));*/
        //float falloff   = (1.0 - sstep(samplelength, 0.0, rsm_size));
        //float falloff   = 1.0 - expf(-samplelength * pi * rcp(2.0));
            //falloff    *= (1.0 - sstep(samplelength, rsm_size*0.8, rsm_size));
        float falloff   = rcp(0.2 + pow2(samplelength * rcp(rsm_size)));
            falloff    *= (1.0 - sstep(samplelength, rsm_size*0.8, rsm_size));

        if (falloff < 1e-4) continue;

        float diffuse   = max(sqrt(dot(shadownormal, samplepos_norm)), 0.0);

        if (diffuse <= 0.0) continue;

        vec3 normal     = texture(shadowcolor1, pos_warped.xy).xyz * 2.0 - 1.0;
            normal      = -normal;

        float diff_bounce = max(sqrt(dot(normal, samplepos_norm)), 0.0);

        if (diff_bounce <= 0.0) continue;

        vec4 x          = texture(shadowcolor0, pos_warped.xy);

        if (x.a < 1.0 && x.a > 0.9) x.rgb = vec3(0.35, 0.74, 1.0) * 0.75;

        float shadow_skylight = x.a * 2.0 - 1.0;
        float bleeding_mask = shadow_skylight - skylight;
            bleeding_mask   = pow6(bleeding_mask);
            bleeding_mask   = saturate(0.05 * rcp(max(bleeding_mask, 0.05)));

        vec3 lsample    = to_linear(x.rgb);

        if (lsample != lsample) lsample = vec3(1.0);

        illum  += lsample * falloff * diffuse * diff_bounce * bleeding_mask;

        sampled = true;
    }

    return illum * rcp(weight);
}