/*
Copyright (C) 2019 RRe36

All Rights Reserved unless otherwise explicitly stated.


By downloading this you have agreed to the license and terms of use.
These can be found inside the included license-file or here: https://rre36.github.io/kappa_shader_web/license/

Violating these terms may be penalized with actions according to the Digital Millennium Copyright Act (DMCA), the Information Society Directive and/or similar laws depending on your country.
*/


#ifdef g_solid
vec3 get_shadowcoord(vec3 viewpos, const float bias, out float warp) {  //shadow 2d
    vec3 pos    = viewpos;
        pos     = viewMAD(gbufferModelViewInverse, pos);
        pos    += vec3(bias)*lightvec;
    float a    = length(pos);
        pos     = viewMAD(shadowModelView, pos);
        pos     = projMAD(shadowProjection, pos);
        pos.z  *= 0.2;
        pos.z  -= 0.0012*(saturate(a/256.0));

        warp    = 1.0;
        pos.xy  = warp_shadowmap(pos.xy, warp);

    return pos*0.5+0.5;
}

vec3 get_shadowcoord(vec3 viewpos, const float bias, out float warp, out float cd) {
    vec3 pos    = viewpos;
        pos     = viewMAD(gbufferModelViewInverse, pos);
        pos    += vec3(bias)*lightvec;
    float a    = length(pos);
        pos     = viewMAD(shadowModelView, pos);
        pos     = projMAD(shadowProjection, pos);
        pos.z  *= 0.2;
        pos.z  -= 0.0012*(saturate(a/256.0));

        warp    = 1.0;
        pos.xy  = warp_shadowmap(pos.xy, warp);

    vec3 p2     = pos;
        p2.xy  *= warp;
        cd      = -projMAD(shadowProjectionInverse, p2).z;

    return pos*0.5+0.5;
}
#endif

float get_shadow(float depth, float cd) {
    return 1.0 - saturate((cd - depth * 256.0 - 0.0086) * 25.0);
}
float get_shadowtex(sampler2D tex, vec2 coord) {
    return texture(tex, coord).x;
}
float get_shadowtex_cubic(sampler2D tex, vec2 coord) {
    return textureBicubic(tex, coord).x;
}
float shadowT2D(sampler2D tex, vec2 pos, float cd) {
    return get_shadow(texture(tex, pos).x, cd);
}

float get_softshadow(sampler2D tex, vec2 pos, float cd) {
    float step  = rcp(float(shadowMapResolution));
    float n     = dither_bluenoise()*pi;
    vec2 noise  = vec2(cos(n), sin(n));
    vec2 offset = noise*step;

    float s0    = shadowT2D(tex, pos, cd)*0.5;
    float s1    = shadowT2D(tex, pos+offset, cd);
    float s2    = shadowT2D(tex, pos-offset, cd);

    return saturate((s0 + s1 + s2)*0.4);
}

float get_softshadow_hq(sampler2D tex, vec2 pos, float cd) {
    float step  = rcp(float(shadowMapResolution));
    float n     = dither_bluenoise()*pi;
    vec2 noise  = vec2(cos(n), sin(n));
    vec2 offset = noise*step;

    float s0    = shadowT2D(tex, pos, cd);
    float s1    = shadowT2D(tex, pos+offset, cd);
    float s2    = shadowT2D(tex, pos-offset, cd);
    float s3    = shadowT2D(tex, pos+offset*2.0, cd)*0.5;
    float s4    = shadowT2D(tex, pos-offset*2.0, cd)*0.5;

    return (s0 + s1 + s2 + s3 + s4)/4.0;
}

float get_softshadow(sampler2DShadow tex, vec3 pos) {
    float step  = rcp(float(shadowMapResolution));
    float n     = dither_bluenoise()*pi;
    vec2 noise  = vec2(cos(n), sin(n));
    vec3 offset = vec3(noise, 0.0)*step;

    float s0    = shadow2D(tex, pos).x*0.5;
    float s1    = shadow2D(tex, pos+offset).x;
    float s2    = shadow2D(tex, pos-offset).x;

    return saturate((s0 + s1 + s2)*0.4);
}

float get_softshadow_hq(sampler2DShadow tex, vec3 pos) {
    float step  = rcp(float(shadowMapResolution));
    float n     = dither_bluenoise()*pi;
    vec2 noise  = vec2(cos(n), sin(n));
    vec3 offset = vec3(noise, 0.0)*step;

    float s0    = shadow2D(tex, pos).x;
    float s1    = shadow2D(tex, pos+offset).x;
    float s2    = shadow2D(tex, pos-offset).x;
    float s3    = shadow2D(tex, pos+offset*2.0).x*0.5;
    float s4    = shadow2D(tex, pos-offset*2.0).x*0.5;

    return (s0 + s1 + s2 + s3 + s4)/4.0;
}

vec3 get_shadowcol(sampler2D tex, vec2 coord) {
    vec4 x  = texture(tex, coord);

    if (x.a < 1.0 && x.a > 0.9) x.a = 0.0;

    return mix(vec3(1.0), x.rgb, x.a);
}

vec3 get_shadowcol(sampler2D tex, vec2 coord, float lmap) {
    vec4 x  = texture(tex, coord);

    #ifdef water_caustic
        if (x.a < 1.0 && x.a > 0.9) {
        x.a = 1.0;

        float caustic = lin_step(x.r, 0.5, 1.0);

        vec3 c  = mix(vec3(0.35, 0.74, 1.0)*0.75, vec3(0.72, 0.91, 1.0)*3.0, caustic);

        float blend = (1.0-pow2(sstep(lmap, 0.0, 0.93))*0.97);

        x.rgb   = vec3(1.0);

        x.rgb   = mix(x.rgb, c, blend);
        }
    #endif

    return mix(vec3(1.0), x.rgb, x.a);
}

#include "/lib/light/filter.glsl"

#ifdef g_solid
void get_ldirect(out float shadow, out vec3 shadowcol, bool diffLit, vec3 viewpos, float lmap) {
    const float bias    = 0.08*(2048.0/shadowMapResolution);
    float warp          = 1.0;
    float cd            = 0.0;
    shadow              = 1.0;
    shadowcol           = vec3(1.0);

    if (diffLit) {
        vec3 pos        = get_shadowcoord(viewpos, bias, warp, cd);

        #if shadow_mode == 0
            float s0        = get_softshadow(shadowtex0, pos.xy, cd);
            float s1        = get_softshadow(shadowtex1, pos.xy, cd);
            bool translucent = distance(s0, s1)>0.2;

            shadow          = s1;

            if (translucent) {
                shadowcol   = get_shadowcol(shadowcolor0, pos.xy, lmap);
            }
            shadowcol       = to_linear(shadowcol);
        #elif shadow_mode == 1
            float s0        = get_softshadow_hq(shadowtex0, pos.xy, cd);
            float s1        = get_softshadow_hq(shadowtex1, pos.xy, cd);

            bool translucent = distance(s0, s1)>0.2;

            shadow          = s1;

            if (translucent) {
                shadowcol   = get_shadowcol(shadowcolor0, pos.xy, lmap);
            }
            shadowcol       = to_linear(shadowcol);
        #elif shadow_mode == 2
            float sigma     = get_vps_sigma(shadowtex1, pos, cd, warp);

            get_shadow30(pos.xy, cd, sigma, shadow, shadowcol);
        #endif
    }
}

#ifdef subsurface_enabled

    #include "subsurface.glsl"

    void get_ldirect_sss(out float shadow, out vec3 shadowcol, out vec3 subsurface, bool diffLit, vec3 viewpos, float lmap, vec3 albedo, float intensity) {
        const float bias    = 0.08*(2048.0/shadowMapResolution);
        float warp          = 1.0;
        float cd            = 0.0;
        shadow              = 1.0;
        shadowcol           = vec3(1.0);

        subsurface          = vec3(0.0);

        vec3 pos        = get_shadowcoord(viewpos, bias, warp, cd);
        if (diffLit) {
            #if shadow_mode == 0
                float s0        = get_softshadow(shadowtex0, pos.xy, cd);
                float s1        = get_softshadow(shadowtex1, pos.xy, cd);
                bool translucent = distance(s0, s1)>0.2;

                shadow          = s1;

                if (translucent) {
                    shadowcol   = get_shadowcol(shadowcolor0, pos.xy, lmap);
                }
                shadowcol       = to_linear(shadowcol);
            #elif shadow_mode == 1
                float s0        = get_softshadow_hq(shadowtex0, pos.xy, cd);
                float s1        = get_softshadow_hq(shadowtex1, pos.xy, cd);

                bool translucent = distance(s0, s1)>0.2;

                shadow          = s1;

                if (translucent) {
                    shadowcol   = get_shadowcol(shadowcolor0, pos.xy, lmap);
                }
                shadowcol       = to_linear(shadowcol);
            #elif shadow_mode == 2
                float sigma     = get_vps_sigma(shadowtex1, pos, cd, warp);

                get_shadow30(pos.xy, cd, sigma, shadow, shadowcol);
            #endif
        } else {
            shadow = 0.0;
        }

        if (shadow < 0.99) {
            float phase     = dot(normalize(viewpos), lightvecView);
                phase       = pow6(phase * 0.5 + 0.5) * 0.6 + 0.5;
            subsurface  = get_subsurface_shadow(shadowtex1, pos.xy, cd, warp, albedo) * intensity * (1.0 - shadow) * phase;
        }
    }
#endif

#else
void get_ldirect(out float shadow, out vec3 shadowcol, bool diffLit) {
    shadow              = 1.0;
    shadowcol           = vec3(1.0);

    if (diffLit) {
        vec3 pos        = pos_shadow;
        float cd        = cd_shadow;
        #if shadow_mode == 0
            float s0        = get_softshadow(shadowtex0, pos.xy, cd);
            float s1        = get_softshadow(shadowtex1, pos.xy, cd);
        #else
            float s0        = get_softshadow_hq(shadowtex0, pos.xy, cd);
            float s1        = get_softshadow_hq(shadowtex1, pos.xy, cd);
        #endif

        bool translucent = distance(s0, s1)>0.2;

        shadow          = s1;

        if (translucent) {
            shadowcol   = get_shadowcol(shadowcolor0, pos.xy);
        }
        shadowcol       = to_linear(shadowcol);
    }
}
#endif