/*
Copyright (C) 2019 RRe36

All Rights Reserved unless otherwise explicitly stated.


By downloading this you have agreed to the license and terms of use.
These can be found inside the included license-file or here: https://rre36.github.io/kappa_shader_web/license/

Violating these terms may be penalized with actions according to the Digital Millennium Copyright Act (DMCA), the Information Society Directive and/or similar laws depending on your country.
*/


const float gauss9w[9] = float[9] (
     0.0779, 0.12325, 0.0779,
    0.12325, 0.1954,  0.12225,
     0.0779, 0.12325, 0.0779
);

const vec2 gauss9o[9] = vec2[9] (
    vec2(1.0, 1.0), vec2(0.0, 1.0), vec2(-1.0, 1.0),
    vec2(1.0, 0.0), vec2(0.0, 0.0), vec2(-1.0, 0.0),
    vec2(1.0, -1.0), vec2(0.0, -1.0), vec2(-1.0, -1.0)
);

float shadow_cd_sss(float depth, float cd) {
    return 1.0 - saturate((cd - depth * 256.0 - 0.0086) * 4.0);
}
float shadowT2D_sss(sampler2D tex, vec2 pos, float cd) {
    return shadow_cd_sss(texture(tex, pos).x, cd);
}

float get_softshadow_sss(sampler2D tex, vec2 pos, float cd, float warp) {
    float step  = 0.0002 * rcp(warp);
    float n     = dither_bluenoise() * pi;
    vec2 noise  = vec2(cos(n), sin(n));
    vec2 offset = noise*step;

    float shadow = 0.0;

    for (int i = 0; i<9; i++) {
        vec2 bcoord = pos + gauss9o[i]*step+offset;
        shadow += shadowT2D_sss(tex, bcoord, cd);
    }

    return shadow * rcp(9.0);
}

vec3 get_subsurface_shadow(sampler2D tex, vec2 pos, float cd, float warp, vec3 albedo) {
    float shadow = get_softshadow_sss(tex, pos, cd, warp); 
        shadow  = pow2(shadow);

    vec3 albedo_norm    = normalize(albedo) * (v3avg(albedo) * 0.5 + 0.5);

    vec3 scatter_col    = mix(albedo_norm * normalize(albedo), mix(albedo_norm, vec3(0.8), pow2(shadow)*0.7), (shadow)) * shadow;

    return scatter_col;
}