/*
Copyright (C) 2019 RRe36

All Rights Reserved unless otherwise explicitly stated.


By downloading this you have agreed to the license and terms of use.
These can be found inside the included license-file or here: https://rre36.github.io/kappa_shader_web/license/

Violating these terms may be penalized with actions according to the Digital Millennium Copyright Act (DMCA), the Information Society Directive and/or similar laws depending on your country.
*/

#include "/lib/common.glsl"
#include "/lib/util/srgb.glsl"

#define INFO 0  //[0]

#define aces_mode 1     //[0 1 2] 0-fast, 1-approx, 2-rrt

in vec2 coord;

uniform sampler2D colortex0;
uniform sampler2D colortex3;
uniform sampler2D colortex4;

uniform sampler2D noisetex;

uniform int frameCounter;

uniform float nightVision;
uniform float frameTimeCounter;

uniform vec2 pixelSize;
uniform vec2 viewSize;

#include "/lib/frag/academy_color.glsl"

float bayer2  (vec2 c) { c = 0.5 * floor(c); return fract(1.5 * fract(c.y) + c.x); }
float bayer4  (vec2 c) { return 0.25 * bayer2 (0.5 * c) + bayer2(c); }
float bayer8  (vec2 c) { return 0.25 * bayer4 (0.5 * c) + bayer2(c); }
float bayer16 (vec2 c) { return 0.25 * bayer8 (0.5 * c) + bayer2(c); }

vec3 dither_image(vec3 color) {
    int bits    = 255;

    vec3 c_dither   = color;
        c_dither   *= bits;
        c_dither   += bayer16(gl_FragCoord.xy);

    return round(c_dither)/bits;
}

vec3 purkinje(vec3 hdr) {
    const vec3 reseponse = vec3(0.15, 0.50, 0.35);

    vec3 desat_color    = vec3(0.4, 0.7, 1.0);
        desat_color     = mix(desat_color, vec3(0.5, 0.9, 1.0), nightVision);

    float desat = dot(hdr, reseponse) * (1.0 + nightVision);
    hdr         = mix(hdr, vec3(desat)*desat_color, (1.0-sstep(desat, 0.0, 0.0072 * purkinje_sensitivity * (1.0 + nightVision))));

    vec2 noisecoord = coord*viewSize;
    float anim  = frameTimeCounter*8*256;

    vec3 noise  = vec3(0.0);
        noise.r = texture(noisetex, floor(noisecoord+anim*1.8)*rcp(256.0)).x;
        noise.g = texture(noisetex, floor(noisecoord+vec2(-anim, anim)*1.2)*rcp(256.0)).x;
        noise.b = texture(noisetex, floor(noisecoord+vec2(anim, -anim)*1.4)*rcp(256.0)).x;

        hdr    += noise * sqrt(1.0-sstep(desat, 0.00, 0.008)) * 0.001;

    return hdr;
}

/* ------ color grading utilities ------ */

vec3 rgb_luma(vec3 x) {
    x.r *= colorlum_r;
    x.g *= colorlum_g;
    x.b *= colorlum_b;

    return x;
}

/* --- post tonemap color grading --- */
vec3 gammacurve(vec3 x) {
    return pow(x, vec3(gamma_curve));
}

vec3 vibrance_saturation(vec3 color) {
    float lum   = dot(color, lumacoeff_rec709);
    float mn    = min(min(color.r, color.g), color.b);
    float mx    = max(max(color.r, color.g), color.b);
    float sat   = (1.0 - (mx-mn)) * (1.0-mx) * lum * 5.0;
    vec3 light  = vec3((mn + mx) / 2.0);

    color   = mix(color, mix(light, color, vibrance_int), sat);

    color   = mix(color, light, (1.0-light) * (1.0-vibrance_int) / 2.0 * abs(vibrance_int));

    color   = mix(vec3(lum), color, saturation_int);

    return color;
}
vec3 brightness_contrast(vec3 color) {
    return (color - 0.5) * constrast_int + 0.5 + brightness_int;
}

#include "/lib/util/bicubic.glsl"

vec4 texture_sharp(sampler2D tex, vec2 uv, const float w) {
    vec2 res    = textureSize(tex, 0);

    vec2 pixelSize = rcp(res);

    float weight = 0.0;

    float corner_weight         = -0.5*w;
    float edge_weight           = 0.46*w;
    const float center_weight   = 1.4;

    vec4 tl     = texture(tex, uv + vec2( 1.0,  1.0)*pixelSize)*corner_weight;    weight += corner_weight;
    vec4 tc     = texture(tex, uv + vec2( 0.0,  1.0)*pixelSize)*edge_weight;    weight += edge_weight;
    vec4 tr     = texture(tex, uv + vec2(-1.0,  1.0)*pixelSize)*corner_weight;    weight += corner_weight;

    vec4 ml     = texture(tex, uv + vec2( 1.0,  0.0)*pixelSize)*edge_weight;    weight += edge_weight;
    vec4 mc     = texture(tex, uv)                  *center_weight;    weight += center_weight;
    vec4 mr     = texture(tex, uv + vec2(-1.0,  0.0)*pixelSize)*edge_weight;    weight += edge_weight;

    vec4 bl     = texture(tex, uv + vec2( 1.0, -1.0)*pixelSize)*corner_weight;    weight += corner_weight;
    vec4 bc     = texture(tex, uv + vec2( 0.0, -1.0)*pixelSize)*edge_weight;    weight += edge_weight;
    vec4 br     = texture(tex, uv + vec2(-1.0, -1.0)*pixelSize)*corner_weight;    weight += corner_weight;

    float norm  = 1.0/weight;

    vec4 col    = (tl+tc+tr+ml+mc+mr+bl+bc+br)*norm;

    if (col.x < 0.0 || col.y < 0.0 || col.z < 0.0) col = mc;

    return col;
}

void main() {
    vec3 scene_hdr  = vec3(0.0);

    #ifdef upscale_sharpen
    if (MC_RENDER_QUALITY > 0.9) {
        scene_hdr  = stex(colortex0).rgb;
    } else if (MC_RENDER_QUALITY < 0.6) {
        scene_hdr = texture_sharp(colortex0, coord, 1.0).rgb;
    } else {
        scene_hdr = texture_sharp(colortex0, coord, 0.66).rgb;
    }
    #else
    if (MC_RENDER_QUALITY > 0.9) {
        scene_hdr  = stex(colortex0).rgb;
    } else {
        scene_hdr  = textureBicubic(colortex0, coord).rgb;
    }
    #endif

    #ifdef bloom_enabled
    vec2 cres       = max(viewSize, vec2(1920.0, 1080.0));

    float bloom_int = 0.012 + max(stex(colortex4).a, 1.0)*0.0025;

    #if dim == -1
        bloom_int  *= 1.3;
    #endif

        scene_hdr  += texture(colortex3, coord/cres*vec2(1920.0, 1080.0)*0.5).rgb*bloom_int*bloom_intensity;  //apply bloom
    #endif

    #ifdef do_colorgrading
        scene_hdr   = rgb_luma(scene_hdr);
    #endif

    #ifndef dim
        scene_hdr   = purkinje(scene_hdr);
    #endif

    #ifdef manual_exposure_enabled
        scene_hdr  *= rcp(manual_exposure);
    #else
        scene_hdr  *= stex(colortex4).a;
    #endif

        //if (bool(greaterThan(sqrt(scene_hdr), vec3(0.9)))) scene_hdr.r = 2.0;

        //if (!bool(greaterThan(sqrt(scene_hdr), vec3(0.1)))) scene_hdr.g = 2.0;

    #if aces_mode == 0
        vec3 scene_sdr  = academy_simplified(scene_hdr);
    #elif aces_mode == 1
        vec3 scene_sdr  = academy_color_encoding_system(scene_hdr);
    #elif aces_mode == 2
        vec3 scene_sdr  = academy_color_encoding_system_rrt(scene_hdr);
    #endif

        //if (coord.x > 0.5) scene_sdr  = academy_simplified(scene_hdr);

    #ifdef do_colorgrading
        scene_sdr   = brightness_contrast(scene_sdr);
        scene_sdr   = vibrance_saturation(scene_sdr);
        scene_sdr   = gammacurve(scene_sdr);
    #endif

    scene_sdr   = dither_image(scene_sdr);

    gl_FragColor    = vec4(scene_sdr, 1.0);
}