#version 400 compatibility

/*
Copyright (C) 2019 RRe36

All Rights Reserved unless otherwise explicitly stated.


By downloading this you have agreed to the license and terms of use.
These can be found inside the included license-file or here: https://rre36.github.io/kappa_shader_web/license/

Violating these terms may be penalized with actions according to the Digital Millennium Copyright Act (DMCA), the Information Society Directive and/or similar laws depending on your country.
*/

#include "/lib/common.glsl"
#include "/lib/util/encoders.glsl"

in vec2 coord;

uniform sampler2D colortex0;
uniform sampler2D colortex1;
uniform sampler2D colortex2;
uniform sampler2D colortex3;
uniform sampler2D colortex5;
uniform sampler2D colortex6;

uniform sampler2D depthtex0;
uniform sampler2D depthtex1;

uniform sampler2D noisetex;

uniform int frameCounter;
uniform int isEyeInWater;

uniform float far;
uniform float near;
uniform float wetness;

uniform vec2 viewSize;

uniform vec3 cameraPosition;
uniform vec3 upvecView;

uniform mat4 gbufferModelView, gbufferModelViewInverse;
uniform mat4 gbufferProjection, gbufferProjectionInverse;

vec3 decode3x8(float a){
    int bf = int(a*65535.);
    return vec3(bf%32, (bf>>5)%64, bf>>11) / vec3(31,63,31);
}

int decodemat_id16(float x) {
    return int(x*65535.0);
}

vec3 blend_translucencies(vec3 scenecol, vec4 translucents, vec3 albedo) {
    vec3 color  = scenecol;
        color  *= mix(vec3(1.0), albedo, translucents.a);
        color   = mix(color, translucents.rgb, translucents.a);

    return color;
}

vec3 apply_fogdata(vec3 scenecol, mat2x3 data) {
    return scenecol.rgb*data[1] + data[0];
}

float depth_lin(float depth) {
    return (2.0*near) / (far+near-depth * (far-near));
}

vec3 view_screenspace(vec3 viewpos) {
    return ((projMAD(gbufferProjection, viewpos) / -viewpos.z)) * 0.5 + 0.5;
}

vec3 view_screenspace2(vec3 viewpos) {
    return projMAD(gbufferProjection, viewpos) * 0.5 + 0.5;
}

vec3 screen_viewspace(vec3 screenpos, mat4 projInv) {
    screenpos   = screenpos*2.0-1.0;

    vec3 viewpos    = vec3(vec2(projInv[0].x, projInv[1].y)*screenpos.xy + projInv[3].xy, projInv[3].z);
        viewpos    /= projInv[2].w*screenpos.z + projInv[3].w;
    
    return viewpos;
}

vec3 screen_viewspace(vec3 screenpos) {
    return screen_viewspace(screenpos, gbufferProjectionInverse);
}

float screen_viewspace(float depth, mat4 projInv) {
    depth   = depth * 2.0 - 1.0;
    return projInv[3].z / (projInv[2].w * depth + projInv[3].w);
}

vec3 view_scenespace(vec3 viewpos, mat4 mvInv) {
    return viewMAD(mvInv, viewpos);
}

vec3 view_scenespace(vec3 viewpos) {
    return view_scenespace(viewpos, gbufferModelViewInverse);
}

#include "/lib/frag/bluenoise.glsl"

#ifdef reflections_enabled
//based on robobo1221's shaders because my old ssr is shit
vec4 ssrTrace(vec3 rdir, vec3 vpos, vec3 screenpos, float dither, float nDotV, const int steps, const int refine) {
    //const int steps     = 16;
    //const int refine    = 5;

    float rlength   = ((vpos.z + rdir.z * far * sqrt(3.0)) > -near) ? (-near - vpos.z) / rdir.z : far * sqrt(3.0);
    vec3 dir        = normalize(view_screenspace(rdir * rlength + vpos) - screenpos);
        dir.xy      = normalize(dir.xy);

    float maxlength = rcp(float(steps));
    float minlength = maxlength*0.05;

    float steplength = mix(minlength, maxlength, (max(nDotV, 0.0)));
    float stepweight = 1.0 / abs(dir.z);

    vec3 pos        = screenpos + dir*steplength*(dither + 0.5);

    float depth     = texelFetch(depthtex1, ivec2(pos.xy * viewSize), 0).x;
    bool ray_hit     = false;

    int i       = steps;
    int j       = refine;

    while (--i > 0) {
        steplength = clamp((depth - pos.z) * stepweight, minlength, maxlength);
        pos += dir*steplength;
        depth = texelFetch(depthtex1, ivec2(pos.xy * viewSize), 0).x;

        if (saturate(pos) != pos) return vec4(0.0);

        if (depth <= pos.z) break;
    }
    float mdepth    = depth;

    vec3 refpos     = pos;
    float refdepth  = depth;

    while (--j > 0) {
        refpos      = dir * clamp((depth - pos.z)*stepweight, -steplength, steplength) + pos;
        refdepth    = texelFetch(depthtex1, ivec2(refpos.xy * viewSize), 0).x;
        bool rayhit = refdepth <= refpos.z;
        if (rayhit) ray_hit = true;

        pos         = rayhit ? refpos : pos;
        depth       = rayhit ? refdepth : depth;

        steplength *= 0.5;
    }

    float sdepth    = texture(depthtex1, pos.xy).x;

    if (sdepth >= 1.0) return vec4(0.0);

    bool visible    = abs(pos.z - mdepth) * min(stepweight, 400.0) <= maxlength;

    return visible ? vec4(texture(colortex0, pos.xy).rgb, 1.0) : vec4(0.0);
}
vec4 ssrTrace(vec3 rdir, vec3 vpos, vec3 screenpos, float dither, float nDotV) {
    return ssrTrace(rdir, vpos, screenpos, dither, nDotV, 16, 5);
}

float fresnel_schlick(vec3 dir, vec3 normal) {
    vec3 halfdir    = normalize(normal + dir);

    float cosine    = dot(halfdir, dir);
    float fresnel   = max(cosine, 0.0);

    return pow3(fresnel);
}

#include "/lib/atmos/project.glsl"

vec2 unproject_sphere(vec3 dir) {
    vec2 lonlat     = vec2(atan(-dir.x, dir.z), acos(dir.y));
    return lonlat * vec2(rcp(tau), rcp(pi)) + vec2(0.5, 0.0);
}

vec4 glossy_reflections(vec3 viewpos, vec3 viewdir, vec3 viewnormal, float depth, float nDotV, float dither, float mult) {
    vec3 rdir       = (reflect(viewdir, viewnormal));

    #ifdef ssr_enabled
        vec4 ssr        = ssrTrace(rdir, viewpos, vec3(coord, depth), dither, nDotV);
    #else
        const vec4 ssr  = vec4(0.0);
    #endif

    vec3 sdir   = normalize(mat3(gbufferModelViewInverse)*rdir);

        mult   *= saturate(1.0 - dot(-upvecView, viewnormal)) * 0.9 + 0.1;

    vec2 spherecoord = unproject_sphere(sdir);
        spherecoord = spherecoord*rcp(SKYREF_LOD) + vec2(0.55);

    vec3 reflection = texture(colortex5, spherecoord).rgb*mult;
        reflection  = mix(reflection, ssr.rgb, ssr.a);

    return vec4(reflection, max(mult, ssr.a));
}

/*
const vec4 hashscale4   = vec4(443.897, 441.423, 437.195, 444.129);

vec4 hash42(float i, vec3 viewpos) {
    vec2 p  = viewpos.xy + (frameCounter%8)*0 - i;
	vec4 p4 = fract(p.xyxy * hashscale4);
	    p4 += dot(p4, p4.wzxy + 19.19);

	return fract((p4.xxyz + p4.yzzw) * p4.zywx);
}

vec3 rough_normal(vec4 noise, float roughness, vec3 normal) {
    noise.xyz   = normalize(cross(normal, noise.xyz));

    return normalize(noise.xyz * (roughness * noise.w * rcp(1.0 - noise.w)) + normal);
}*/

/*
These two functions used for rough reflections are based on zombye's spectrum shaders
https://github.com/zombye/spectrum
*/

mat3 get_rotation_mat(vec3 x, vec3 y) {
	float cosine = dot(x, y);
	vec3 axis = cross(y, x);

	float tmp = 1.0 / dot(axis, axis);
	      tmp = tmp - tmp * cosine;
	vec3 tmpv = axis * tmp;

	return mat3(
		axis.x * tmpv.x + cosine, axis.x * tmpv.y - axis.z, axis.x * tmpv.z + axis.y,
		axis.y * tmpv.x + axis.z, axis.y * tmpv.y + cosine, axis.y * tmpv.z - axis.x,
		axis.z * tmpv.x - axis.y, axis.z * tmpv.y + axis.x, axis.z * tmpv.z + cosine
	);
}
vec3 ggx_facet_dist(vec3 viewdir, float roughness, vec2 xy) {
	/*
    GGX VNDF sampling
	http://www.jcgt.org/published/0007/04/01/
    */

    viewdir     = normalize(vec3(roughness * viewdir.xy, viewdir.z));

    float clsq  = dot(viewdir.xy, viewdir.xy);
    vec3 T1     = vec3(clsq > 0.0 ? vec2(-viewdir.y, viewdir.x) * inversesqrt(clsq) : vec2(1.0, 0.0), 0.0);
    vec3 T2     = vec3(-T1.y * viewdir.z, viewdir.z * T1.x, viewdir.x * T1.y - T1.x * viewdir.y);

	float r     = sqrt(xy.x);
	float phi   = tau * xy.y;
	float t1    = r * cos(phi);
	float a     = saturate(1.0 - t1 * t1);
	float t2    = mix(sqrt(a), r * sin(phi), 0.5 + 0.5 * viewdir.z);

	vec3 normal_h = t1 * T1 + t2 * T2 + sqrt(saturate(a - t2 * t2)) * viewdir;

	return normalize(vec3(roughness * normal_h.xy, normal_h.z));
}

vec4 rough_reflections(vec3 viewpos, vec3 viewdir, vec3 viewnormal, float depth, float nDotV, float dither, float mult, float roughness) {
    if (roughness > 0.9) return vec4(0.0);

    float rfade     = 1.0 - lin_step(roughness, 0.6, 0.9);
    
    int steps = 2 + int(min(roughness * 3.0, 2.0));
    float rstep = 1.0 / float(steps);

    vec3 total      = vec3(0.0);
    float ssr_alpha = 0.0;

    mat3 rot    = get_rotation_mat(vec3(0, 0, 1), viewnormal);
    vec3 tangent_v = viewdir * rot;

    vec2 xy     = vec2(0.0);
    vec3 nrm    = vec3(0.0);
    vec3 rdir   = vec3(0.0);
    vec4 ssr    = vec4(0.0);
    vec3 sdir   = vec3(0.0);
    vec2 spherecoord = vec2(0.0);
    vec3 reflection = vec3(0.0);

    float sum_mult  = 0.0;

    for (int i = 0; i<steps; ++i) {
        //vec3 nrm    = vec3(0.0, 0.0, 1.0);
        //    nrm     = rough_normal(hash42(float(i) * rstep, viewpos), roughness, viewnormal);

            xy     = vec2(fract((i + dither) * pow2(32.0) * phi), (i + dither) * rstep);
            nrm    = rot * ggx_facet_dist(-tangent_v, roughness, xy); 

            rdir   = reflect(viewdir, nrm);

        #ifdef ssr_enabled
            if (roughness >= 0.75) {
                ssr = ssrTrace(rdir, viewpos, vec3(coord, depth), dither, nDotV, 6, 4);
            } else if (roughness >= 0.5) {
                ssr = ssrTrace(rdir, viewpos, vec3(coord, depth), dither, nDotV, 8, 4);
            } else if (roughness >= 0.25) {
                ssr = ssrTrace(rdir, viewpos, vec3(coord, depth), dither, nDotV, 8, 4);
            } else if (roughness >= 0.12) {
                ssr = ssrTrace(rdir, viewpos, vec3(coord, depth), dither, nDotV, 10, 5);
            } else {
                ssr = ssrTrace(rdir, viewpos, vec3(coord, depth), dither, nDotV, 12, 5);
            }
        #endif

            sdir   = normalize(mat3(gbufferModelViewInverse)*rdir);

            float smult = saturate(1.0 - dot(-upvecView, nrm)) * 0.9 + 0.1;
            sum_mult   += smult;

            spherecoord = unproject_sphere(sdir);
            spherecoord = spherecoord*rcp(SKYREF_LOD) + vec2(0.55);

            reflection  = texture(colortex5, spherecoord).rgb * mult * smult;
            reflection  = mix(reflection, ssr.rgb, ssr.a);

        total  += reflection;
        ssr_alpha += ssr.a;
    }

    mult   *= sum_mult * rstep;
    total  *= rstep;
    ssr_alpha *= rstep;

    return vec4(total, max(mult, ssr_alpha)) * rfade;
}

#define noise_2d_only
#include "/lib/frag/noise.glsl"
#include "/lib/frag/wetness.glsl"

vec3 fresnel_conductor(vec3 eta, vec3 etak, float cos_theta) {
    float cosTheta2 = cos_theta * cos_theta;
    float sinTheta2 = 1 - cosTheta2;
    vec3 eta2 = eta * eta;
    vec3 etak2 = etak * etak;

    vec3 t0 = eta2 - etak2 - sinTheta2;
    vec3 a2plusb2 = sqrt(t0 * t0 + 4 * eta2 * etak2);
    vec3 t1 = a2plusb2 + cosTheta2;
    vec3 a = sqrt(0.5f * (a2plusb2 + t0));
    vec3 t2 = 2 * a * cos_theta;
    vec3 Rs = (t1 - t2) / (t1 + t2);

    vec3 t3 = cosTheta2 * a2plusb2 + sinTheta2 * sinTheta2;
    vec3 t4 = t2 * sinTheta2;   
    vec3 Rp = Rs * (t3 - t4) / (t3 + t4);

    return saturate(0.5 * (Rp + Rs));
}

#ifdef labpbr_enabled
#include "/lib/frag/labPBR.glsl"

void reflection(inout vec3 scenecol, vec3 viewnormal, vec3 scenenormal, float depth, vec2 lmap, int mat_id, vec2 pbr_tex, bool translucent, vec3 metal_hue) {
    vec4 spectex    = vec4(decode2x8(pbr_tex.x), decode2x8(pbr_tex.y));
    vec4 mat_data   = vec4(1.0, 0.02, 0.0, 0.0);
    float emission  = 0.0;
    bool is_metal   = false;
    mat2x3 metal_eta = mat2x3(0.0);
    bool hardcoded_metal = false;

    #ifdef labpbr_enabled
        decode_lab(spectex, mat_data, emission, is_metal, metal_eta, hardcoded_metal);
    #endif

    //mat_data.x  = 0.05;

    if (translucent) mat_data.zw = vec2(0.0);

    bool water      = mat_id == 102;

    vec3 screenpos  = vec3(coord, depth);
    vec3 viewpos    = screen_viewspace(vec3(coord, depth));
    vec3 viewdir    = normalize(viewpos);
    vec3 scenepos   = view_scenespace(viewpos);
    //vec3 scenedir   = normalize(scenepos);

    if (wetness > 0.0) {
        float block_wetness = get_wetness(lmap.y, scenenormal, scenepos + cameraPosition);

        mat_data.x  = mix(mat_data.x, 0.04, block_wetness);
        mat_data.y  = mix(mat_data.y, 0.02, block_wetness);
    }
    
    float nDotV     = saturate(dot(viewnormal, viewdir));

    float dither    = dither_bluenoise();

    float f0        = water ? 0.02 : mat_data.y;
        if (is_metal) f0 = 0.35;

    vec3 fresnel    = vec3(0.0);
    if (hardcoded_metal) fresnel = fresnel_conductor(metal_eta[0], metal_eta[1], saturate(dot(viewnormal, -viewdir)));
    else fresnel = vec3(max(fresnel_schlick(viewdir, viewnormal), f0));

    float fmult     = pow2(sstep(lmap.y, 0.15, 0.95));

    vec4 reflection = vec4(0.0);

    #ifdef rough_reflections_enabled
        if (mat_data.x < 0.002 || water) reflection  = glossy_reflections(viewpos, viewdir, viewnormal, depth, nDotV, dither, fmult);
        else reflection  = rough_reflections(viewpos, viewdir, viewnormal, depth, nDotV, dither, fmult, mat_data.x);
    #else
        reflection  = glossy_reflections(viewpos, viewdir, viewnormal, depth, nDotV, dither, fmult);
        reflection.a *= pow2(1.0 - lin_step(mat_data.x, 0.0, 0.9));
    #endif

    if (hardcoded_metal) scenecol = mix(scenecol.rgb, reflection.rgb * pow2(fresnel), reflection.a);
    else if (is_metal) scenecol = mix(scenecol.rgb, reflection.rgb * metal_hue, reflection.a);
    else scenecol = mix(scenecol.rgb, reflection.rgb, fresnel*reflection.a);

    //scenecol = reflection.rgb;
}
#else
void reflection(inout vec3 scenecol, in vec3 viewnormal, in vec3 scenenormal, in float depth, vec2 lmap, int mat_id) {
    vec3 screenpos  = vec3(coord, depth);
    vec3 viewpos    = screen_viewspace(vec3(coord, depth));
    vec3 viewdir    = normalize(viewpos);
    vec3 scenepos   = view_scenespace(viewpos);
    //vec3 scenedir   = normalize(scenepos);

    float block_wetness = 0.0;

    if (wetness > 0.01) {
        block_wetness = get_wetness(lmap.y, scenenormal, scenepos + cameraPosition) * wetness;
    }
    
    if (!(block_wetness > 0.01 || mat_id == 102 || mat_id == 103)) return;

    float nDotV     = saturate(dot(viewnormal, viewdir));

    float dither    = dither_bluenoise();

    float fresnel   = max(fresnel_schlick(viewdir, viewnormal), 0.02);
        if (mat_id == 103) fresnel *= 0.35;
        if (block_wetness > 0.01 && !(mat_id == 102 || mat_id == 103)) fresnel *= sqrt(block_wetness);
        //fresnel     = 1.0;

    float fmult     = pow2(sstep(lmap.y, 0.15, 0.95));

    vec4 reflection = glossy_reflections(viewpos, viewdir, viewnormal, depth, nDotV, dither, fmult);

    scenecol        = mix(scenecol.rgb, reflection.rgb, fresnel*reflection.a);
    //scenecol        = compute_ssr(viewnormal, screenpos, viewpos, viewdir, rdir, nDotV, dither).rgb;
}
#endif

#endif

mat2x3 textureBilateralDouble(sampler2D tex1, sampler2D tex2, sampler2D depth, const float lod, float fdepth, const vec2 offset, vec2 coord) {
    mat2x3 data = mat2x3(0.0);
    float sum   = 0.0;
    ivec2 posD  = ivec2(coord*viewSize);
    ivec2 posT  = ivec2((coord*rcp(lod)+offset)*viewSize);
    vec3 zmult  = vec3((far*near)*2.0, far+near, far-near);
        fdepth  = depth_lin(fdepth);

    ivec2 tcDepth = ivec2(0);
    float dsample = 0.0;
    float w     = 0.0;
    ivec2 ct    = ivec2(0);
    
    for (int i = -1; i<2; i++) {
        for (int j = -1; j<2; j++) {
                tcDepth = posD + ivec2(vec2(i, j)*lod);
                dsample = depth_lin(texelFetch(depth, tcDepth, 0).x);
                w     = abs(dsample-fdepth)*zmult.x<1.0 ? 1.0 : 1e-5;
                ct    = posT + ivec2(i, j);
            data[0]    += texelFetch(tex1, ct, 0).rgb*w;
            data[1]    += texelFetch(tex2, ct, 0).rgb*w;
            sum        += w;
        }
    }
    data *= rcp(sum);

    return data;
}

void main() {
    vec4 scenecol   = stex(colortex0);
    vec4 tex1       = stex(colortex1);
    vec4 tex2       = stex(colortex2);
    vec4 tex3       = stex(colortex3);

    vec2 scenelmap  = decode2x8(tex1.y);

    vec3 scenenormal = decodeNormal(decode2x8(tex1.x));
    vec3 viewnormal = normalize(mat3(gbufferModelView) * scenenormal);

    vec3 translucent_albedo = pow2(decode3x8(tex2.g));

    float scenedepth0 = stex(depthtex0).r;
    float scenedepth1 = stex(depthtex1).r;

    int mat_id      = decodemat_id16(tex2.x);

    bool water      = mat_id == 102;

    bool translucent = (scenedepth0<scenedepth1);

    vec2 water_coord = coord;

    #ifdef water_refraction
        if (water) {
            vec3 viewpos0     = screen_viewspace(vec3(coord, scenedepth0));
            vec3 refracted_pos = refract(normalize(viewpos0), viewnormal, rcp(1.166)) + viewpos0;
            vec2 refracted_coord = view_screenspace(refracted_pos).xy;
            vec2 refracted_coord1 = mix(coord, refracted_coord, 0.9);
            vec2 refracted_coord2 = mix(coord, refracted_coord, 1.1);

            float d1    = texture(depthtex1, refracted_coord).x;
            bool valid_refract   = (d1 > scenedepth0) && (saturate(refracted_coord) == refracted_coord);

            if (valid_refract) {
                vec2 coord      = refracted_coord;
                water_coord     = refracted_coord;
                scenecol.r      = textureLod(colortex0, refracted_coord1, 0).r;
                scenecol.g      = textureLod(colortex0, refracted_coord, 0).g;
                scenecol.b      = textureLod(colortex0, refracted_coord2, 0).b;
                scenedepth1     = stex(depthtex1).x;
            }
        }
    #endif

    #if (defined vwater_enabled || defined vfog_translucents)
        mat2x3 atmos1;
            atmos1[0] = vec3(0.0);
            atmos1[1] = vec3(1.0);

        if (translucent || isEyeInWater == 1) {
            vec2 fcoord_atmos = saturate(coord*rcp(2.02) + vec2(0.0, 0.5));

            #ifdef vwater_bilateral
                atmos1      = textureBilateralDouble(colortex5, colortex6, depthtex0, 2.02, scenedepth0, vec2(0.0, 0.5), water_coord);
            #else
                atmos1[0]   = texture(colortex5, fcoord_atmos).rgb;
                atmos1[1]   = texture(colortex6, fcoord_atmos).rgb;
            #endif
        }
        
        if (translucent && isEyeInWater == 0) scenecol.rgb = apply_fogdata(scenecol.rgb, atmos1);
    #endif

    #ifdef vfog_enabled
        vec2 fcoord_atmos = saturate(coord*0.5 + vec2(0.5, 0.0));

        mat2x3 atmos;
            atmos[0] = vec3(0.0);
            atmos[1] = vec3(1.0);

        #ifdef vfog_bilateral
            atmos       = textureBilateralDouble(colortex5, colortex6, depthtex0, 2, scenedepth0, vec2(0.5, 0.0), coord);
        #else
            atmos[0]      = texture(colortex5, fcoord_atmos).rgb;
            atmos[1]      = texture(colortex6, fcoord_atmos).rgb;
        #endif

        if (isEyeInWater == 1) scenecol.rgb = apply_fogdata(scenecol.rgb, atmos);
    #endif

    scenecol.rgb    = blend_translucencies(scenecol.rgb, tex3, translucent_albedo);

    #ifdef reflections_enabled
        #ifdef labpbr_enabled
        if (landMask(scenedepth0)) {
            vec3 metal_hue = (decode3x8(tex2.g));
            reflection(scenecol.rgb, viewnormal, scenenormal, scenedepth0, scenelmap, mat_id, tex1.zw, translucent, metal_hue);
        }
        #else
        if (water || mat_id == 103 || (wetness > 0.01 && landMask(scenedepth0))) reflection(scenecol.rgb, viewnormal, scenenormal, scenedepth0, scenelmap, mat_id);
        #endif
    #endif

    #ifdef vfog_enabled
        if (isEyeInWater == 0) scenecol.rgb = apply_fogdata(scenecol.rgb, atmos);
    #endif
    
    #if (defined vwater_enabled || defined vfog_translucents)
        if (isEyeInWater == 1) scenecol.rgb = apply_fogdata(scenecol.rgb, atmos1);
    #endif

    if (mat_id == 3) {
        //if (landMask(scenedepth0)) scenecol.rgb = scenecol.rgb*0.6 + vec3(0.5)*v3avg(scenecol.rgb);
        scenecol.rgb = scenecol.rgb*0.7 + vec3(0.8)*v3avg(scenecol.rgb);
    }

    //if (isnan4(scenecol) || isinf4(scenecol)) scenecol.rgb = vec3(0.0, 40.0, 0.0);

    /*DRAWBUFFERS:05*/
    gl_FragData[0]  = make_drawbuffer(scenecol);
    gl_FragData[1]  = vec4(0.0, 0.0, 0.0, 1.0);
}