#version 400 compatibility

/*
Copyright (C) 2019 RRe36

All Rights Reserved unless otherwise explicitly stated.


By downloading this you have agreed to the license and terms of use.
These can be found inside the included license-file or here: https://rre36.github.io/kappa_shader_web/license/

Violating these terms may be penalized with actions according to the Digital Millennium Copyright Act (DMCA), the Information Society Directive and/or similar laws depending on your country.
*/

#include "/lib/common.glsl"

/*
const int colortex0Format   = RGBA16F;
const int colortex1Format   = RGBA16;
const int colortex2Format   = RG16;
const int colortex3Format   = RGBA16F;
const int colortex4Format   = RGBA16F;
const int colortex5Format   = RGBA16F;
const int colortex6Format   = RGB16F;
const int colortex7Format   = RGBA16;

const vec4 colortex0ClearColor = vec4(0.0, 0.0, 0.0, 1.0);
const vec4 colortex3ClearColor = vec4(0.0, 0.0, 0.0, 0.0);

c0  - 4x16 scene color (full)
c1  - 1x16 encoded normals, 1x16 lightmaps, 2x16 specular texture (gbuffer -> composite2)
c2  - 1x16 matID, 1x16 translucency albedo hue (gbuffer -> composite2)
c3  - clouds (deferred0 -> deferred1), ao and rsm (deferred1 -> deferred3), 4x16 translucencies (water -> composite0), bloom (composite7 -> final)
c4  - temporals (full)
c5  - sky (deferred0 -> composite0), fog (composite -> composite1)
c6  - fog (composite -> composite1)
c7  - rsm accumulation (full)
*/

const int noiseTextureResolution = 256;

in vec2 coord;

flat in vec3 atmos_multiscatter;
flat in vec3 atmos_upcol;

flat in mat4x3 light_color;

uniform vec2 viewSize;
uniform vec2 pixelSize;

uniform sampler2D colortex0;
uniform sampler2D depthtex1;

uniform sampler2D noisetex;
uniform sampler3D depthtex2;

uniform int frameCounter;
uniform int worldTime;

uniform vec4 daytime;
uniform float eyeAltitude;
uniform float frameTimeCounter;
uniform float cloud_bsfade;
uniform float wetness;

uniform vec3 upvec, upvecView;
uniform vec3 sunvec, sunvecView;
uniform vec3 moonvec, moonvecView;
uniform vec3 lightvec;
uniform vec3 cloud_lvec, cloud_lvecView;

uniform vec3 cameraPosition;

uniform mat4 gbufferModelView, gbufferModelViewInverse;
uniform mat4 gbufferProjection, gbufferProjectionInverse;

vec3 screen_viewspace(vec3 screenpos, mat4 projInv) {
    screenpos   = screenpos*2.0-1.0;

    vec3 viewpos    = vec3(vec2(projInv[0].x, projInv[1].y)*screenpos.xy + projInv[3].xy, projInv[3].z);
        viewpos    /= projInv[2].w*screenpos.z + projInv[3].w;
    
    return viewpos;
}

vec3 screen_viewspace(vec3 screenpos) {
    return screen_viewspace(screenpos, gbufferProjectionInverse);
}

vec3 view_scenespace(vec3 viewpos, mat4 mvInv) {
    return viewMAD(mvInv, viewpos);
}

vec3 view_scenespace(vec3 viewpos) {
    return view_scenespace(viewpos, gbufferModelViewInverse);
}

#include "/lib/atmos/phase.glsl"

#include "/lib/atmos/asky.glsl"

#include "/lib/atmos/project.glsl"

#include "/lib/frag/bluenoise.glsl"

#include "/lib/frag/noise.glsl"

vec2 rotate_coord(vec2 pos, const float angle) {
    return vec2(cos(angle)*pos.x + sin(angle)*pos.y, 
                cos(angle)*pos.y - sin(angle)*pos.x);
}

float fbm(vec3 pos, vec3 offset, const float persistence, const float scale, const int octaves) {
    float n     = 0.0;
    float a     = 1.0;
    vec3 shift  = offset;

    for (int i = 0; i<octaves; ++i) {
        n      += value_3d(pos + shift)*a;
        pos.xz  = rotate_coord(pos.xz, pi*0.33);
        pos    *= scale;
        a      *= persistence;
    }

    return n;
}

float cloud_mie_dumb(float cos_theta, float g) {
    float sqG   = pow2(g);
    float a     = (1.0-sqG) / (2.0 + sqG);
    float b     = (1.0 + pow2(cos_theta)) / (1.0 + sqG - 2.0*g*cos_theta);

    return max(1.5 * a*b + g*cos_theta, 0.0)*rcp(pi);
}
float cloud_mie(float x, float g) {
    float mie   = 1.0 + pow2(g) - 2.0*g*x;
        mie     = (1.0 - pow2(g)) / ((4.0*pi) * mie*(mie*0.5+0.5));
    return mie;
}

float vcloud_tick   = frameTimeCounter*0.1;

#include "/lib/atmos/vcloud.glsl"

/*
dummy place for proper cloud lighting with real names

float rho       = 0.99;         //albedo, should be close to 1.0

float sigma_a   = 1.00;         //absorption coeff
float sigma_s   = 0.12;         //scattering coeff, can technically be assumed to be sigma_t since the albedo is close to 1.0
float sigma_t   = 0.12;         //extinction coeff, 0.05-0.12 for cumulus, 0.04-0.06 for stratus

vec4 result     = vec4(0.0, 0.0, 0.0, 1.0);

#define light_steps 6

//inside raymarcher
    float extinction = shape result;
    float extinct_c = max(extinction, 0.0000001);
    float transmittance = exp(-extinction * steplength(?));

    float integral  = (1.0 - 1.0*transmittance) / extinct_c;

    vec3 scatterResult = blah, placeholder, just wanna get a draft of the method;

    result.rgb += result.a * integral * scatterResult;
    result.a   *= transmittance;

//get scattering
    float energy    = exp(-scatter_depth*sigma_a)*phase*powder;


//multiscattering, a <= b
    float sigma_s_d     = sigma_s * pow(a, n);
    float sigma_t_d     = sigma_t * pow(b, n);
    float phase         = phase(vdotl, pow(c, n));

*/

void compute_vc(out mat2x4 data, in vec3 vvec, in vec3 wvec, in vec2 coord) {
    float within    = sstep(eyeAltitude, vcloud_alt-100.0, vcloud_alt) * (1.0-sstep(eyeAltitude, vc_highedge, vc_highedge+100.0));
    bool visible    = wvec.y>0.0;

    if (visible || within>0.0) {
        const float vc_midalt = vcloud_alt+vcloud_depth*0.5;

        bool is_below   = eyeAltitude<vc_midalt;

        vec3 bs     = wvec*((vcloud_alt-eyeAltitude)/wvec.y);
        vec3 ts     = wvec*((vc_highedge-eyeAltitude)/wvec.y);

        if (wvec.y<0.0 && is_below || wvec.y>0.0 && !is_below){
            bs = vec3(0.0);
            ts = vec3(0.0);
        }

        vec3 spos   = is_below ? bs : ts;
        vec3 epos   = is_below ? ts : bs;

            spos    = mix(spos, gbufferModelViewInverse[3].xyz, within);
            epos    = mix(epos, wvec*90000.0, within);

        float dither = dither_bluenoise();

        const float bl  = vcloud_depth/vcloud_samples;
        float stepl     = length((epos-spos)/vcloud_samples);
        float stepcoeff = stepl/bl;
            stepcoeff   = 0.45+clamp(stepcoeff-1.1, 0.0, 4.0)*0.5;
            stepcoeff   = mix(stepcoeff, 10.0, pow2(within));
        int steps       = int(vcloud_samples*stepcoeff);

        vec3 rstep  = (epos-spos)/steps;
        vec3 rpos   = rstep*dither + spos + cameraPosition;
        float rlength = length(rstep);

        vec3 scatter    = vec3(0.0);
        float transmittance = 1.0;
        float fade      = 0.0;
        float fdist     = vcloud_clip+1.0;

        vec3 sunlight   = (worldTime>23000 || worldTime<12900) ? light_color[0] : light_color[3];
        vec3 skylight   = light_color[1]*0.33;
        vec3 bouncelight = (sunlight + light_color[1]) * 0.008 * vec3(0.7, 1.0, 0.6);

        float vdotl     = dot(vvec, cloud_lvecView);

        float pfade     = saturate(cloud_mie(vdotl, 0.65));

        const float sigma_a = 1.00;         //absorption coeff
        const float sigma_s = 0.10;         //scattering coeff, can technically be assumed to be sigma_t since the albedo is close to 1.0
        const float sigma_t = 0.10;         //extinction coeff, 0.05-0.12 for cumulus, 0.04-0.06 for stratus

        //float depth     = 0.0;
        //float stept_sum = 0.0;

        for (int i = 0; i<steps; ++i, rpos += rstep) {
            if (transmittance < 0.05) break;
            if (rpos.y < vcloud_alt || rpos.y > vc_highedge) continue;

            float dist  = distance(rpos, cameraPosition);
            if (dist > vcloud_clip) continue;
            float dfade = saturate(dist/vcloud_clip);
            //if ((1.0-dfade)<0.01) continue;

            float coverage  = vc_coverage(rpos);
            if (coverage < 0.01) continue;

            float density   = vc_shape(rpos, coverage);
            if (density<=0.0) continue;

            float f     = lin_step(dfade, 0.75, 0.99);

            if (fdist>vcloud_clip) {
                fdist   = dist;
                fade    = f;
            } else {
                fdist   = mix(fdist, dist, transmittance);
                fade    = mix(fade, f, transmittance);
            }

            //this part is new and made after the draft somewhere above

            float extinction = density * sigma_t;
            float stept     = expf(-extinction*rlength);
            float integral  = (1.0 - stept) / sigma_t;

            //depth      += dfade * stept;
            //stept_sum  += stept;

            vec3 result_s   = vec3(0.0);

            float directod  = vc_directOD(rpos, 6)*sigma_a;
            float skyod     = vc_skyOD(rpos, 3)*sigma_a;

            float powder    = 1.0 - expf(-density * vc_powder_K);
            float dpowder   = min(powder, 1.0-expf(-directod*2.0));    //because sun od based powder simply adds this little "extra", using min here gives a much more pleasurable result
                dpowder     = mix(dpowder, 1.0, pfade);

            for (int j = 0; j<8; ++j) {
                float n     = float(j);

                float s_d   = sigma_s * pow(0.5, n);    //scatter derivate
                float t_d   = sigma_t * pow(0.5, n);    //transmittance/attentuation derivate
                float phase = cloud_phase(vdotl, pow(0.5, n));  //phase derivate

                result_s.x += expf(-directod*t_d) * phase * dpowder * s_d;
                result_s.y += expf(-skyod*t_d) * powder * s_d;
            }
            result_s.z = vc_bouncelight(rpos) * (powder * 0.5 + 0.5);

            scatter    += result_s * integral * transmittance;

            transmittance *= stept;
        }
        fdist   = max(fdist, 0.0);
        transmittance = lin_step(transmittance, 0.05, 1.0);
        scatter.x *= 2.4;

        vec3 color  = (sunlight*scatter.x) + (skylight*scatter.y) + (bouncelight*scatter.z);

        fade        = saturate(1.0-fade);
        //fade        = pow2(fade);

        /*
        vec3 air        = get_rho(length(wvec) * wvec.y * 0.5 + eyeAltitude + planetRad);
            air         = extinct_mat * air;*/

        vec3 atmosfade  = expf(-extinct_mat * vec3(fdist));
        float skyfade   = expf(-fdist * cloud_atmos_density);

        /*
        vec3 atmosfade  = vec3(1.0);
        float skyfade   = 1.0;

        if (fdist < vcloud_clip) {
                stept_sum   = max(stept_sum, 1e-16);
                depth       = depth * rcp(stept_sum) * vcloud_clip;

            vec3 air        = get_rho(length(wvec) * wvec.y * 0.5 + eyeAltitude + planetRad);
                air         = extinct_mat * air;

                atmosfade   = expf(-air * depth);
                skyfade     = expf(-depth * cloud_atmos_density);
        }*/

        data[0]     = vec4(color, transmittance);
        data[1]     = vec4(atmosfade * skyfade * fade, 1.0);
    } else {
        data[0]     = vec4(0.0, 0.0, 0.0, 0.0);
        data[1]     = vec4(0.0);
    }
}

float max_depth3x3(sampler2D depthtex, vec2 coord, vec2 px) {

    float tl    = texture(depthtex, coord + vec2(-px.x, -px.y)).x;
    float tc    = texture(depthtex, coord + vec2(0.0, -px.y)).x;
    float tr    = texture(depthtex, coord + vec2(px.x, -px.y)).x;
    float tmin  = max(tl, max(tc, tr));

    float ml    = texture(depthtex, coord + vec2(-px.x, 0.0)).x;
    float mc    = texture(depthtex, coord).x;
    float mr    = texture(depthtex, coord + vec2(px.x, 0.0)).x;
    float mmin  = max(ml, max(mc, mr));

    float bl    = texture(depthtex, coord + vec2(-px.x, px.y)).x;
    float bc    = texture(depthtex, coord + vec2(0.0, px.y)).x;
    float br    = texture(depthtex, coord + vec2(px.x, px.y)).x;
    float bmin  = max(bl, max(bc, br));

    return max(tmin, max(mmin, bmin));
}

void main() {
    mat2x4 cloud_data;

    const float cLOD    = sqrt(CLOUD_RENDER_LOD);

    vec2 scalecoord  = (coord-vec2(1.0-rcp(cLOD), 0.0))*cLOD;

    float d     = max_depth3x3(depthtex1, scalecoord, pixelSize*cLOD);

    #ifdef vcloud_enabled
        if (clamp(scalecoord, -0.003, 1.003) == scalecoord && !landMask(d)) {
            scalecoord      = clamp(scalecoord, 0.0, 1.0);
            vec3 viewpos    = screen_viewspace(vec3(scalecoord, 1.0));
            vec3 viewvec    = normalize(viewpos);
            vec3 scenepos   = view_scenespace(viewpos);
            vec3 svec       = normalize(scenepos);

            compute_vc(cloud_data, viewvec, svec, scalecoord);
        } else {
            cloud_data[0] = vec4(0.0, 0.0, 0.0, 0.0);
            cloud_data[1] = vec4(0.0);
        }
    #else
        cloud_data[0] = vec4(0.0, 0.0, 0.0, 1.0);
        cloud_data[1] = vec4(0.0);
    #endif

    if(!(coord.y >= exp2(-SKY_RENDER_LOD) || coord.x >= exp2(-SKY_RENDER_LOD))) {
        vec3 sceneDir = unprojectSky(coord);
        vec3 skycol   = scatter(sceneDir, mat2x3(sunvec, moonvec), atmos_multiscatter);
        cloud_data[1] = vec4(skycol, 1.0);
    }

    #ifdef sun_transmittance_hq
    vec2 coord_t    = coord-vec2(0.0, 0.3);

    if(!(coord_t.y >= exp2(-SKY_RENDER_LOD) || coord_t.x >= exp2(-SKY_RENDER_LOD)) && (coord_t.y>=0.0)) {
        vec3 sceneDir = unprojectSky(coord_t);
        vec3 skycol   = air_transmittance(vec3(0.0, planetRad*0.9998 + eyeAltitude, 0.0), sceneDir, 6);
        //vec3 skycol   = scatter(sceneDir, sunvec, moonvec, 16);
        cloud_data[1] = vec4(skycol, 1.0);
    }
    #endif

    /*DRAWBUFFERS:35*/
    gl_FragData[0]  = clamp_drawbuffer(cloud_data[0]);
    gl_FragData[1]  = clamp_drawbuffer(cloud_data[1]);
}