#version 400 compatibility

/*
Copyright (C) 2019 RRe36

All Rights Reserved unless otherwise explicitly stated.


By downloading this you have agreed to the license and terms of use.
These can be found inside the included license-file or here: https://rre36.github.io/kappa_shader_web/license/

Violating these terms may be penalized with actions according to the Digital Millennium Copyright Act (DMCA), the Information Society Directive and/or similar laws depending on your country.
*/

#include "/lib/common.glsl"

in vec2 coord;

uniform sampler2D colortex3;

uniform sampler2D depthtex1;

uniform float far, near;

uniform vec2 pixelSize, viewSize;

float depth_lin(float depth) {
    return (2.0*near) / (far+near-depth * (far-near));
}

vec4 filter_spatial_gi(sampler2D tex, sampler2D depth, vec2 coord) {
    vec2 d_pos  = coord * 2.0;
    vec2 c_pos  = coord;

    vec4 center_col     = texture(tex, c_pos);

    float center_depth  = texture(depth, d_pos).x;
        center_depth    = depth_lin(center_depth) * far;

    vec4 sumcol     = vec4(0.0);
    float sumweight = 0.0;
    
    const float depth_thresh    = 0.4;
    const float depth_rsigma    = ircp((depth_thresh * depth_thresh) * 2.0);
    const float depth_rpi       = ircp(pi) * ircp(depth_thresh);

    const float color_thresh    = 0.25;
    const float color_rsigma    = ircp((color_thresh * color_thresh) * 2.0);
    const float color_rpi       = ircp(pi) * ircp(color_thresh);

    const int sigma     = 8;
    const float rsigma  = 0.5 / (sigma * sigma);

    for (int x = -sigma; x<=sigma; x++) {
        for (int y = -sigma; y<=sigma; y++) {
            vec2 delta_pos  = vec2(x, y) * pixelSize;

            float blur  = expf(-dot(delta_pos, delta_pos) * rsigma) * rsigma;

            vec2 depth_pos  = d_pos + delta_pos * 2.0;
            float curr_depth = depth_lin(texture(depth, d_pos).x) * far;
            float depth_weight = pow2(1.0 - sstep(distance(curr_depth, center_depth), depth_thresh / 4, depth_thresh));

            vec2 curr_pos   = c_pos + delta_pos;
            vec4 curr_col   = texture(tex, curr_pos);

            float weight    = depth_weight;

            if (depth_weight > 0.95) {
                vec3 delta_col  = curr_col.rgb - center_col.rgb;
                float color_weight = expf(-dot(delta_col, delta_col) * color_rsigma) * color_rpi * blur;
                weight  = color_weight;
            }

            sumcol     += curr_col * weight;
            sumweight  += 1.0 * weight;
        }
    }

    sumcol /= max(sumweight, 1e-16);

    return sumcol;
}

float max_depth3x3(sampler2D depthtex, vec2 coord, vec2 px) {

    float tl    = texture(depthtex, coord + vec2(-px.x, -px.y)).x;
    float tc    = texture(depthtex, coord + vec2(0.0, -px.y)).x;
    float tr    = texture(depthtex, coord + vec2(px.x, -px.y)).x;
    float tmin  = max(tl, max(tc, tr));

    float ml    = texture(depthtex, coord + vec2(-px.x, 0.0)).x;
    float mc    = texture(depthtex, coord).x;
    float mr    = texture(depthtex, coord + vec2(px.x, 0.0)).x;
    float mmin  = max(ml, max(mc, mr));

    float bl    = texture(depthtex, coord + vec2(-px.x, px.y)).x;
    float bc    = texture(depthtex, coord + vec2(0.0, px.y)).x;
    float br    = texture(depthtex, coord + vec2(px.x, px.y)).x;
    float bmin  = max(bl, max(bc, br));

    return max(tmin, max(mmin, bmin));
}

void main() {
    vec4 tex3   = stex(colortex3);

    #ifdef rsm_spatial_dualpass
    vec2 ao_coord   = (coord)*2.0;

    float d     = max_depth3x3(depthtex1, ao_coord, pixelSize*sqrt(2.0));

    if (clamp(ao_coord, -0.003, 1.003) == ao_coord && landMask(d)) {
        vec2 acoord      = saturate(ao_coord);
        float scenedepth = texture(depthtex1, acoord).x;

        vec3 gi     = filter_spatial_gi(colortex3, depthtex1, coord).rgb;
            //gi      = texture(colortex3, coord).rgb;

        tex3.rgb    = gi;
    }
    #endif

    /*DRAWBUFFERS:3*/
    gl_FragData[0]  = tex3;
}