#version 400 compatibility

/*
Copyright (C) 2019 RRe36

All Rights Reserved unless otherwise explicitly stated.


By downloading this you have agreed to the license and terms of use.
These can be found inside the included license-file or here: https://rre36.github.io/kappa_shader_web/license/

Violating these terms may be penalized with actions according to the Digital Millennium Copyright Act (DMCA), the Information Society Directive and/or similar laws depending on your country.
*/

#include "/lib/common.glsl"
#include "/lib/util/encoders.glsl"
#include "/lib/util/srgb.glsl"

const int shadowMapResolution   = 2560;     //[512 1024 1536 2048 2560 3072 3584 4096 6144 8192 16384]
const float shadowDistance      = 128.0;

const bool shadowHardwareFiltering = false;

in vec2 coord;

flat in float light_flip;

flat in mat4x3 light_color;

flat in mat4x3 IBL_light;

uniform sampler2D colortex0;
uniform sampler2D colortex1;
uniform sampler2D colortex2;
uniform sampler2D colortex3;
uniform sampler2D colortex5;

uniform sampler2D noisetex;

uniform sampler2D depthtex1;

uniform sampler2D shadowtex0;
uniform sampler2D shadowtex1;
uniform sampler2D shadowcolor0;
uniform sampler2D shadowcolor1;

uniform int frameCounter;

uniform float aspectRatio;
uniform float far, near;
uniform float sunAngle;
uniform float viewHeight, viewWidth;
uniform float wetness;

uniform vec2 taaOffset;
uniform vec2 viewSize;

uniform vec3 cameraPosition;
uniform vec3 lightvec, lightvecView;
uniform vec3 shadowLightPosition;

uniform vec4 daytime;

uniform mat4 gbufferModelView, gbufferModelViewInverse;
uniform mat4 gbufferProjection, gbufferProjectionInverse;
uniform mat4 shadowModelView, shadowModelViewInverse;
uniform mat4 shadowProjection, shadowProjectionInverse;

vec3 screen_viewspace(vec3 screenpos, mat4 projInv) {
    screenpos   = screenpos*2.0-1.0;

    #ifdef taa_enabled
        screenpos.xy -= taaOffset;
    #endif

    vec3 viewpos    = vec3(vec2(projInv[0].x, projInv[1].y)*screenpos.xy + projInv[3].xy, projInv[3].z);
        viewpos    /= projInv[2].w*screenpos.z + projInv[3].w;
    
    return viewpos;
}

vec3 screen_viewspace(vec3 screenpos) {
    return screen_viewspace(screenpos, gbufferProjectionInverse);
}

vec3 view_scenespace(vec3 viewpos, mat4 mvInv) {
    return viewMAD(mvInv, viewpos);
}

vec3 view_scenespace(vec3 viewpos) {
    return view_scenespace(viewpos, gbufferModelViewInverse);
}

float depth_lin(float depth) {
    return (2.0*near) / (far+near-depth * (far-near));
}

#include "/lib/light/diffuse.glsl"

#include "/lib/light/warp.glsl"

#include "/lib/frag/bluenoise.glsl"

#define g_solid

#include "/lib/util/bicubic.glsl"

#include "/lib/light/shadow.glsl"

vec3 get_lblock(vec3 lcol, float lmap) {
    return pow5(lmap)*lcol;
}

float get_specGGX(vec3 normal, vec3 svec, vec2 material) {
    float f0  = material.y;
    float roughness = pow2(material.x);

    vec3 h      = lightvec - svec;
    float hn    = inversesqrt(dot(h, h));
    float hDotL = saturate(dot(h, lightvec)*hn);
    float hDotN = saturate(dot(h, normal)*hn);
    float nDotL = saturate(dot(normal, lightvec));
    float denom = (hDotN * roughness - hDotN) * hDotN + 1.0;
    float D     = roughness / (pi * denom * denom);
    float F     = f0 + (1.0-f0) * exp2((-5.55473*hDotL-6.98316)*hDotL);
    float k2    = 0.25 * roughness;

    return nDotL * D * F / (hDotL * hDotL * (1.0-k2) + k2);
}

vec3 get_IBL(vec3 scenenormal, vec3 skylight, mat4x3 IBL_light, float lmap) {
    float weight_0  = max(0.0, dot(scenenormal, vec3(1.0, 0.0, 0.0)));
    float weight_1  = max(0.0, dot(scenenormal, vec3(0.0, 0.0, 1.0)));
    float weight_2  = max(0.0, dot(scenenormal, vec3(-1.0, 0.0, 0.0)));
    float weight_3  = max(0.0, dot(scenenormal, vec3(0.0, 0.0, -1.0)));
    float weight_4  = max(0.0, dot(scenenormal, vec3(0.0, 1.0, 0.0)));
    float weight_5  = max(0.0, dot(scenenormal, vec3(0.0, -1.0, 0.0)));

    vec3 direct_col     = sunAngle<0.5 ? light_color[0] : light_color[3];
        direct_col     *= light_flip;

    float bounceweight = max(0.0, dot(vec3(0.0, -1.0, 0.0), -lightvec));

    vec3 IBL    = vec3(0.0);
        IBL    += IBL_light[0] * weight_0;
        IBL    += IBL_light[1] * weight_1;
        IBL    += IBL_light[2] * weight_2;
        IBL    += IBL_light[3] * weight_3;        
        IBL    += skylight * weight_4;
        //IBL     = IBL * 0.75 + skylight * 0.25;
        IBL     = mix(skylight * (1.0 - weight_5 * 0.35), IBL, lmap * 0.75);
        IBL    += skylight * weight_5 * 0.0375 * lmap;
        IBL    += direct_col * weight_5 * 0.0625 * lmap * bounceweight;

    return IBL;
}

vec3 get_light(vec3 scenecol, vec3 normal, vec3 viewpos, vec3 scenepos, vec2 lmap, vec3 gi, float ao, int matID, vec4 mat_data, bool is_metal) {
    float shadow    = 1.0;
    vec3 shadowcol  = vec3(1.0);

    #ifdef labpbr_enabled
    float roughness = mat_data.x;
    #else
    const float roughness = 1.0;
    #endif

    float diff      = get_diffOrenNayar(mat3(gbufferModelView)*normal, -normalize(viewpos), lightvecView, roughness);
        diff        = matID != 2 ? diff : diff*0.3+0.7;

    #ifdef subsurface_enabled
        vec3 subsurface = vec3(0.0);

        #ifdef labpbr_enabled
            float sss_int = mat_data.w;
        #else
            float sss_int = float(matID == 4);
        #endif

            sss_int *= 1.0 - sstep(length(scenepos), 64.0, 128.0);

        if (sss_int > 0.0) get_ldirect_sss(shadow, shadowcol, subsurface, diff>0.0, viewpos, lmap.y, scenecol, sss_int);
        else get_ldirect(shadow, shadowcol, diff>0.0, viewpos, lmap.y);

    #else

        get_ldirect(shadow, shadowcol, diff>0.0, viewpos, lmap.y);

    #endif

    lmap.y          = pow3(lmap.y);

    float diff_lit  = min(diff, shadow);

    #ifdef pom_shadows_enabled
        if (diff_lit > 0.0) diff_lit *= stex(colortex2).y;
    #endif

    vec3 direct_col     = sunAngle<0.5 ? light_color[0] : light_color[3];
    vec3 direct_light   = diff_lit*shadowcol*direct_col;

    #ifdef subsurface_enabled
        direct_light   += subsurface*shadowcol*direct_col;
    #endif

        direct_light   *= ao*0.5+0.5;
        direct_light   *= light_flip;

    vec3 ibl    = get_IBL(normal, light_color[1], IBL_light, lmap.y);
        //if (matID == 2) ibl = light_color[1];

    vec3 indirect_light = lmap.y*ibl + pow2(lmap.y) * direct_col * 0.02 * light_flip;
        indirect_light += vec3(0.5, 0.7, 1.0)*0.01*minlight_luma;
        indirect_light *= ao;
        indirect_light += gi * direct_col * light_flip;

    #ifdef lightleak_workaround
        direct_light *= sstep(lmap.y, 0.04, 0.1);
    #endif

    vec3 result     = direct_light + indirect_light;
        result     += get_lblock(light_color[2], lmap.x)*ao;

    #ifdef labpbr_enabled
    vec3 specular   = direct_col * get_specGGX(normal, normalize(mat3(gbufferModelViewInverse) * viewpos), mat_data.xy);
        specular   *= diff_lit * light_flip;
    if (is_metal) {     //this needs to be replaced with proper metal specular
        specular   *= scenecol.rgb;
        scenecol.rgb *= scenecol.rgb;
    }

    #if DEBUG_VIEW==1
    scenecol.rgb    = vec3(1.0);
    #endif

    return scenecol * result + specular;
    
    #else

    #if DEBUG_VIEW==1
    scenecol.rgb    = vec3(1.0);
    #endif

    return scenecol * result;

    #endif
    //return vec3(diff_lit);
}

int decodeMatID16(float x) {
    return int(x*65535.0);
}

vec4 textureBilateral(sampler2D tex, sampler2D depth, const int lod, float fdepth) {
    vec4 data   = vec4(0.0);
    float sum   = 0.0;
    ivec2 posD  = ivec2(coord*viewSize);
    ivec2 posT  = ivec2(coord*viewSize*rcp(float(lod)));
    vec3 zmult  = vec3((far*near)*2.0, far+near, far-near);
        fdepth  = depth_lin(fdepth);
    //ivec2 p     = (ivec2(gl_FragCoord.xy + frameCounter) % 2 - 1)*0;
    
    for (int i = -2; i<3; i++) {
        for (int j = -2; j<3; j++) {
            ivec2 tcDepth = posD + ivec2(i, j)*lod;
            float dsample = depth_lin(texelFetch(depth, tcDepth, 0).x);
            float w     = abs(dsample-fdepth)*zmult.z<1.0 ? 1.0 : 1e-5;
            ivec2 ct    = posT + ivec2(i, j);
            data       += texelFetch(tex, ct, 0)*w;
            sum        += w;
        }
    }
    data *= rcp(sum);

    return data;
}

#include "/lib/frag/labPBR.glsl"
#define noise_2d_only
#include "/lib/frag/noise.glsl"
#include "/lib/frag/wetness.glsl"

vec2 unproject_sphere(vec3 dir) {
    vec2 lonlat     = vec2(atan(-dir.x, dir.z), acos(dir.y));
    return lonlat * vec2(rcp(tau), rcp(pi)) + vec2(0.5, 0.0);
}

#include "/lib/atmos/phase.glsl"

vec3 compute_vcloud_ssgr(vec3 svec, vec3 vvec) {
    float mie       = hg_mie(dot(svec, lightvec), 0.74);

    vec3 nFrag      = -vvec;
    vec3 sgVec      = normalize(lightvecView+nFrag);
    float sunGrad   = 1.0-dot(sgVec, nFrag);
    float sunGlow   = lin_step(sunGrad, 0.2, 0.8);
        sunGlow     = (sunGlow) * mie;
        sunGlow    *= 1.0 - sstep(cameraPosition.y, vcloud_alt * 0.75, vcloud_alt * 0.95);

    const float cLOD    = sqrt(CLOUD_RENDER_LOD);

    float density   = 0.66*daytime.x + 0.14*daytime.y + 0.3*daytime.z + 0.4*daytime.w;
        density     = mix(density, 1.6, wetness);
        density    *= cloud_ssgr_density;

    if (sunGlow > 0.0) {
        vec3 lightcol   = sunAngle<0.5 ? light_color[0] : light_color[3];
            lightcol   *= light_flip;

        vec4 tpos   = vec4(shadowLightPosition, 1.0) * gbufferProjection;
            tpos.xyz /= tpos.w;
            tpos.xy = tpos.xy/tpos.z;
        vec2 lightpos   = tpos.xy * 0.5 + 0.5;
        float truepos   = shadowLightPosition.z/abs(shadowLightPosition.z);

        float sunpos    = abs(dot(svec, lightvec));
        //float decay     = pow(sunpos, 30.0)+pow(sunpos, 16.0)*0.8+pow2(sunpos)*0.125;

        float dither    = dither_bluenoise();

        vec2 dcoord     = (lightpos - coord) * cloud_ssgr_length * 0.75;
            dcoord     *= rcp(float(cloud_ssgr_samples));
        vec2 tcoord     = coord - dcoord*dither;

        vec3 godrays    = vec3(0.0);

        float visible   = 1.0-lin_step(length(tpos.xy), 0.85, 1.25);
            //decay      *= visible;

        //if (clamp(lightpos, -0.05, 1.05) != lightpos) decay = 0.0;

        if (visible > 0.0 && truepos < 1.0) {
            for (int i = 0; i<cloud_ssgr_samples; i++) {
                tcoord += dcoord;

                vec3 svec   = screen_viewspace(vec3(tcoord, 1.0));
                    svec    = view_scenespace(svec);
                    svec    = normalize(svec + vec3(0.0, cameraPosition.y * rcp(30.0) * (1.0 - abs(normalize(svec).y)), 0.0));

                //float temp   = texture(colortex3, saturate(tcoord)*rcp(cLOD)+vec2(1.0-rcp(cLOD), 0.0), 0).a;
                float temp  = texture(colortex5, unproject_sphere(svec)*rcp(SKYREF_LOD) + vec2(0.55)).a;
                godrays    += temp * (1.0 - (float(i)/cloud_ssgr_samples));
            }
            godrays *= rcp(float(cloud_ssgr_samples));
            godrays *= sunGlow * visible;
        }
        return godrays*lightcol*density;
    } else {
        return vec3(0.0);
    }
}

float bayer2e(vec2 a){
    a = floor(a);
    return fract( dot(a, vec2(.5, a.y * .75)) );
}
#define bayer4e(a)   (bayer2e( .5*(a))*.25+bayer2e(a))

#define m vec3(31,63,31)
float encode3x8(vec3 a){
    float dither = bayer4e(gl_FragCoord.xy);
    a += (dither-.5) / m;
    a = saturate(a);
    ivec3 b = ivec3(a*m);
    return float( b.r|(b.g<<5)|(b.b<<11) ) / 65535.;
}
#undef m

void main() {
    vec4 scenecol   = stex(colortex0);
    vec4 tex5       = stex(colortex5);
    float scenedepth = stex(depthtex1).x;

    vec3 viewpos    = screen_viewspace(vec3(coord, scenedepth));
    vec3 scenepos   = view_scenespace(viewpos);

    vec4 return2    = stex(colortex2);

    if (landMask(scenedepth)) {
        vec4 tex1       = stex(colortex1);
        vec2 scenelmap  = decode2x8(tex1.y);
        vec3 scenenormal = decodeNormal(decode2x8(tex1.x));

        vec4 tex2       = stex(colortex2);
            return2     = tex2;

        int matID       = decodeMatID16(tex2.x);

        float ao        = pow2(scenecol.a)*0.66+0.34;

        #if (defined ambientOcclusion_enabled || defined rsm_enabled)
            vec4 tex3   = textureBilateral(colortex3, depthtex1, 2, scenedepth);
                tex3.rgb *= rcp(10.0);
                ao     *= tex3.a;
        #else
            const vec4 tex3 = vec4(0.0, 0.0, 0.0, 1.0);
        #endif

        vec4 spectex    = vec4(decode2x8(tex1.z), decode2x8(tex1.w));
        vec4 mat_data   = vec4(1.0, 0.02, 0.6, 0.0);
        float emission  = 0.0;
        bool is_metal   = false;

        float block_wetness = 0.0;

        if (wetness > 0.01) block_wetness = get_wetness(scenelmap.y, scenenormal, scenepos + cameraPosition) * wetness;

        #ifdef labpbr_enabled
        decode_lab(spectex, mat_data, emission, is_metal);
        scenecol.rgb    = compute_porosity(scenecol.rgb, vec3(mat_data.xz, block_wetness));
        if (block_wetness > 0.01) {
            mat_data.x  = mix(mat_data.x, 0.04, block_wetness);
            mat_data.y  = mix(mat_data.y, 0.02, block_wetness);
        }
        vec3 emitted    = scenecol.rgb * emission;
        #else
        scenecol.rgb    = compute_porosity(scenecol.rgb, vec3(mat_data.xz, block_wetness));
        #endif

        if (is_metal) return2.y = encode3x8(normalize(scenecol.rgb) * (v3avg(scenecol.rgb) * 0.5 + 0.5));
        
        scenecol.rgb    = get_light(scenecol.rgb, scenenormal, viewpos, scenepos, scenelmap, tex3.rgb, ao, matID, mat_data, is_metal);

        #if DEBUG_VIEW==2
        scenecol.rgb    = ao * (sunAngle<0.5 ? light_color[0] : light_color[3]);
        #endif

        #ifdef labpbr_enabled
        scenecol.rgb   += emitted * tau;
        #endif

        //scenecol.rgb    = tex3.rgb;

        //scenecol.rgb    = vec3(block_wetness);

        //scenecol.rgb    = (scenenormal * 0.5 + 0.5) * v3avg(sunAngle<0.5 ? light_color[0] : light_color[3]) * 0.5;
    } else {
        #ifdef cloud_ssgr_enabled
            scenecol.rgb   += compute_vcloud_ssgr(normalize(scenepos), normalize(viewpos));
        #endif
    }

    //scenecol.rgb = vec3(scenedepth);

    /*DRAWBUFFERS:0235*/
    gl_FragData[0]  = make_drawbuffer(scenecol);
    gl_FragData[1]  = clamp_drawbuffer(return2);
    gl_FragData[2]  = vec4(0.0);
    gl_FragData[3]  = clamp_drawbuffer(tex5);
}